from Algorithms.MOEA_D.random_selection import random_selection
from Algorithms.MOEA_D.neighborhood_weight import neighborhood_weight
from Algorithms.MOEA_D.update_neighbour import update_neighbour
from Algorithms.MOEA_D.update_external_pop import update_external_pop
from Algorithms.MOEA_D.tchebycheff import tchebycheff
from Algorithms.MOEA_D.check_domination import check_domination
from Algorithms.MOEA_D.non_dominated_sort import non_dominated_sort


from Algorithms.Utility.eval_objective_values import eval_objective_values
from Algorithms.Utility.print_elapsed_time import print_elapsed_time
from Algorithms.Utility.mate import mate
from Algorithms.Utility import logData
import config as cg

import time
import numpy as np
import matlab.engine
import array
import json
import os

def moea_d(eng, toolbox, motion, chrom_repr, L_BOUND, U_BOUND, loggerObj):

    elapsed_time = 0
    elapsed_computational_time = 0
    t1 = time.perf_counter()

    # print('====================================================')
    print('Generation: 0')

    pop, n_gen = logData.start_from_temporary_save(loggerObj, toolbox)

    for i in range(len(pop)):
        pop[i].Low = L_BOUND
        pop[i].Upp = U_BOUND
    sim_time = eval_objective_values(pop, eng, motion)

    # Plot objectives through MATLAB
    obj1 = [pop[i].fitness.values[0] for i in range(len(pop))]
    obj2 = [pop[i].fitness.values[1] for i in range(len(pop))]
    eng.plot_objectives(obj1, obj2, nargout=0)

    # Create neighborhood weight (sub-problems)
    weight, B = neighborhood_weight()
    # print(B)
    ref_point = cg.ref_point
    EP        = []

    elapsed_time += time.perf_counter() - t1
    print_elapsed_time(elapsed_time, elapsed_time, sim_time, elapsed_computational_time, \
                                                                                1, cg.n_generations)
    for i in range(len(pop)):
        for j in range(cg.n_objectives):
            if ref_point[j] > pop[i].fitness.values[j]:
                ref_point[j] = pop[i].fitness.values[j]

    # for i in range(len(pop)):
    #     pop[i].g = tchebycheff(pop[i], weight[i], ref_point)
    # check_domination(pop)
    front = non_dominated_sort(pop)
    for ind in front:
        EP += [toolbox.clone(ind)]

    for gen in range(n_gen, cg.n_generations+1):
        sim_time = 0
        t1 = time.perf_counter()
        offs = []
        print('====================================================')
        print('Generation: %d' % gen)
        print('Evaluate population')
        for i in range(len(pop)):
            neighbours= []
            for j in range(len(B[i])):
                neighbours += [pop[B[i][j]]]

            offspring = mate(neighbours, toolbox, random_selection, chrom_repr, 2)
            ind = offspring[np.random.randint(0,high=len(offspring))]
            # for ind in offspring:
            offs += [ind]

        print(len(offs))
        sim_time = eval_objective_values(offs, eng, motion)
        for ind,i in zip(offs,range(len(offs))):
            for j in range(cg.n_objectives):
                if ref_point[j] > ind.fitness.values[j]:
                    ref_point[j] = ind.fitness.values[j]
            update_neighbour(ind, B[i], pop, weight, ref_point, toolbox)

        front = non_dominated_sort(pop)
        for ind in front:
            EP += [toolbox.clone(ind)]

        # check_domination(pop)
        # for i in range(len(pop)):
        #     if not pop[i].dominated:
        #         EP += [toolbox.clone(pop[i])]

        front = non_dominated_sort(EP)
        EP = front

        # check_domination(EP)
        # for i in range(len(EP)):
        #     if EP[i].dominated:
        #         del EP[i]

        # update_external_pop(EP, ind, toolbox)
        t2  = time.perf_counter()
        # print(EP)
        print("The size of the population: ", len(pop))

        elapsed_time += t2 - t1
        elapsed_computational_time += (t2-t1) - sim_time
        print_elapsed_time(elapsed_time, (t2-t1), sim_time, elapsed_computational_time, \
                                                                            gen,cg.n_generations)
        if len(EP) > cg.pop_size:
            for i in range(len(EP)-cg.pop_size):
                del EP[np.random.randint(0,high=len(EP))]
        print("The size of the external population: ", len(EP))


        obj1 = [EP[i].fitness.values[0] for i in range(len(EP))]
        obj2 = [EP[i].fitness.values[1] for i in range(len(EP))]
        eng.plot_objectives(obj1, obj2, nargout=0)

    print("======================= End Of Optimization Run =======================")
    input("Press Enter to continue...")
    return EP
