from Algorithms.NSGA2.binary_tournament_selection import binary_tournament_selection
from Algorithms.NSGA2.create_parent_population import create_parent_population

# from Algorithms.Utility.decode_ind import decode_ind
from Algorithms.Utility.eval_objective_values import eval_objective_values
from Algorithms.Utility.print_elapsed_time import print_elapsed_time
from Algorithms.Utility.non_dominated_sort import non_dominated_sort
from Algorithms.Utility.penalty_function import penalty_function
from Algorithms.Utility.mate import mate
from Algorithms.Utility import logData

import config as cg

import time
import matlab.engine
import array
import os

def nsga2(eng, toolbox, motion, chrom_repr, L_BOUND, U_BOUND, loggerObj):

    elapsed_time = 0
    elapsed_computational_time = 0
    t1 = time.perf_counter()

    pop, n_gen = logData.start_from_temporary_save(loggerObj, toolbox)

    for i in range(len(pop)):
        pop[i].Low = L_BOUND
        pop[i].Upp = U_BOUND
    sim_time = eval_objective_values(pop, eng, motion)
    front =  non_dominated_sort(pop, ranking=True)

    elapsed_time += time.perf_counter() - t1
    print_elapsed_time(elapsed_time, elapsed_time, sim_time, elapsed_computational_time, \
                                                                                1, cg.n_generations)

    for gen in range(n_gen, cg.n_generations+1):

        t1 = time.perf_counter()
        print("=======================================================================")
        print('Generation: %d' % gen)
        print('Evaluate population')

        offspring = mate(pop, toolbox, binary_tournament_selection, chrom_repr, cg.pop_size)
        invalid_ind = [ind for ind in offspring if not ind.fitness.valid]
        sim_time = eval_objective_values(invalid_ind, eng, motion)

        front =  non_dominated_sort(pop + offspring)
        pop = create_parent_population(front)

        front =  non_dominated_sort(pop)
        # Due to penalty, add bad fitness
        violated = penalty_function(pop)
        if violated:
            for i in range(len(front)):
                print("==========VIOLATED===========")
                print("Front %d:" % (i+1))
                for j in range(len(front[i])):
                    print("\trank: %d" % front[i][j].rank)


        obj         = [pop[i].fitness.values for i in range(len(pop))]
        violation   = [pop[i].violations for i in range(len(pop))]

        obj1 = [obj[i][0] for i in range(len(obj))]
        obj2 = [obj[i][1] for i in range(len(obj))]

        t2  = time.perf_counter()
        elapsed_time += t2 - t1
        elapsed_computational_time += (t2-t1) - sim_time
        print_elapsed_time(elapsed_time, (t2-t1), sim_time, elapsed_computational_time, \
                                                                            gen,cg.n_generations)

        logData.save_history(loggerObj, pop, obj)
        if gen == cg.n_generations:
            os.chdir('../results/All')
            print("=======================================================================")
            if input("Do you want to save these results?[Y/N]: ") == 'Y':
                logData.final_save(loggerObj)
        elif gen%1 == 0 and gen < (cg.n_generations-5):
            print("Temporary saved generation: ", str(gen))
            logData.temporary_save(loggerObj)
            eng.plot_objectives(obj1, obj2, nargout=0)


    print("======================= End Of Optimization Run =======================")
    input("Press Enter to continue...")
    return pop

if __name__ == "__main__":
    nsga2()
    input("Press Enter to continue...")
