import config as cg

import ast
import gzip
import json
import os

import zipfile
import zlib

class LoggerObject:
    def __init__(self, method, motion, result_name, info_name, temp_name, zip_name, cx, mx):

        self.result_file = result_name
        self.result_info = info_name
        self.temp_file   = temp_name
        self.zip_file    = zip_name

        self.method     = method
        self.motion     = motion

        self.popHist    = []
        self.objHist    = []

        self.cx         = cx
        self.mx         = mx
        # self.violHist   = []

    def getKey(self):
        # key = ["pop%d" % cg.pop_size, "gen%d" % cg.n_generations, \
        #         "cross%.2f" % cg.crossover_rate, "mut%.2f" % cg.mutation_rate]
        key = ["pop%d" % cg.pop_size, "gen%d" % cg.n_generations, \
                "cross%.2f" % self.cx, "mut%.3f" % self.mx]
        return key

    def storeHistory(self, population, objectives):
        self.popHist    += [population]
        self.objHist    += [objectives]
        # self.violHist   += [violations]

    def loadBinary(self):

        with gzip.open(self.result_file + '.gz', 'rb') as f:
            data = f.read()
            f.close()

        data = data.decode('ascii')
        if data == '':
            data  = {}
        else:
            data  = ast.literal_eval(data)

        return data

    def loadJSON(self):

        with open(self.result_file) as result_data:
            data = json.loads(result_data.read())
            result_data.close()

        return data

    def loadTemporaryFile(self):

        # with gzip.open(self.temp_file + '.gz', 'rb') as f:
            # data = f.read()
        # data = data.decode('ascii')

        data = json.loads(open(self.temp_file).read())

        method   = self.method
        motion   = self.motion
        key      = self.getKey()
        attr     = ["population","objectives","violations"]
        keyExist = False
        # key[1]   = "gen200"

        # if data != '':
            # data = ast.literal_eval(data)
        if method in data:
            if motion in data[method]:
                if key[0] in data[method][motion]:
                    if key[1] in data[method][motion][key[0]]:
                        if key[2] in data[method][motion][key[0]][key[1]]:
                            if key[3] in data[method][motion][key[0]][key[1]][key[2]]:
                                keyExist = True
        if keyExist:
            self.popHist  = data[method][motion][key[0]][key[1]][key[2]][key[3]][attr[0]]
            self.objHist  = data[method][motion][key[0]][key[1]][key[2]][key[3]][attr[1]]
            # self.violHist = data[method][motion][key[0]][key[1]][key[2]][key[3]][attr[2]]
            return len(self.popHist)
        return 1

    def writeToBinary(self, data, file):
        with gzip.open(file+'.gz', 'w') as fp:
            fp.write(str.encode(str(data)))
        fp.close()

    def writeToJson(self, data, file):
        jsonText = '{'
        for method in data:
            jsonText += '\n\t\"' + str(method) + '\":{'
            for motion in data[method]:
                jsonText += '\n\t\t\"' + str(motion) + '\":{'
                for pop in data[method][motion]:
                    jsonText += '\n\t\t\t\"' + str(pop) + '\":{'
                    for gen in data[method][motion][pop]:
                        jsonText += '\n\t\t\t\t\"' + str(gen) + '\":{'
                        for cross in data[method][motion][pop][gen]:
                            jsonText += '\n\t\t\t\t\t\"' + str(cross) + '\":{'
                            for mut in data[method][motion][pop][gen][cross]:
                                results1 = data[method][motion][pop][gen][cross][mut]["population"]
                                results2 = data[method][motion][pop][gen][cross][mut]["objectives"]
                                # results3 = data[method][motion][pop][gen][cross][mut]["violations"]
                                jsonText += '\n\t\t\t\t\t\t\"' + str(mut)  +  '\":{'
                                jsonText += '\n\t\t\t\t\t\t\t\"population\":' + str(results1)
                                jsonText += ',\n\t\t\t\t\t\t\t\"objectives\":'+ str(results2)
                                # jsonText += ',\n\t\t\t\t\t\t\t\"violations\":'+ str(results3)
                                jsonText += '\n\t\t\t\t\t\t},'
                            jsonText = jsonText[:len(jsonText)-2] + '}'
                            jsonText += '\n\t\t\t\t\t},'
                        jsonText = jsonText[:len(jsonText)-2] + '}'
                        jsonText += '\n\t\t\t\t},'
                    jsonText = jsonText[:len(jsonText)-2] + '}'
                    jsonText += '\n\t\t\t},'
                jsonText = jsonText[:len(jsonText)-2] + '}'
                jsonText += '\n\t\t},'
            jsonText = jsonText[:len(jsonText)-2] + '}'
            jsonText += '\n\t},'
        jsonText = jsonText[:len(jsonText)-1] + '\n}'

        with open(file, 'w') as fp:
            fp.write(jsonText)
        fp.close()

    def temporarySave(self):

        method = self.method
        motion = self.motion
        key    = self.getKey()

        data = {
            method:{
                motion:{
                    key[0]:{
                        key[1]:{
                            key[2]:{
                                key[3]:{
                                    "population": self.popHist,
                                    "objectives": self.objHist
                                    # "violations": self.violHist
                                }
                            }
                        }
                    }
                }
            }
            # "nr_of_gen": n_gen
        }
        self.writeToJson(data, self.temp_file)

    def updateInformation(self):

        method      = self.method
        motion      = self.motion
        key         = self.getKey()
        lineUpdated = False

        infoText = ''
        with  open(self.result_info, 'r+') as fp:
            for line in fp:
                keyExist = False
                if line.find(method) > -1:
                    if line.find(motion) > -1:
                        if line.find(key[0]) > -1:
                            if line.find(key[1]) > -1:
                                if line.find(key[2]) > -1:
                                    if line.find(key[3]) > -1:
                                        keyExist = True
                if keyExist and not lineUpdated:
                    i = line.find(':')
                    nData = int(line[:i]) + 1
                    updateLine = str(nData) + ": " + method + ", " + motion + ", " +\
                                    key[0]+", "+key[1]+", "+key[2]+", "+key[3] + '\n'
                    infoText += updateLine
                    lineUpdated = True
                else:
                    infoText += line

            if not lineUpdated:
                infoText += "1: "+ method + ", " + motion + ", " + \
                            key[0]+", "+key[1]+", "+key[2]+", "+key[3]+'\n'
            fp.seek(0)
            fp.write(infoText)
            fp.truncate()
            fp.close()

    def mergeData(self, data, method, motion, key):

        keyExist = False

        if not method in data:
            data[method] = {
                motion: {
                    key[0]:{
                        key[1]:{
                            key[2]:{
                                key[3]:{
                                    "population": [self.popHist],
                                    "objectives": [self.objHist]
                                    # "violations": self.violHist
                                }
                            }
                        }
                    }
                }
            }
        elif not motion in data[method]:
            data[method][motion] = {
                key[0]:{
                    key[1]:{
                        key[2]:{
                            key[3]:{
                                "population": [self.popHist],
                                "objectives": [self.objHist]
                                # "violations": self.violHist
                            }
                        }
                    }
                }
            }

        elif key[0] in data[method][motion]:
            keyExist = 1
            if key[1] in data[method][motion][key[0]]:
                keyExist = 2
                if key[2] in data[method][motion][key[0]][key[1]]:
                    keyExist = 3
                    if key[3] in data[method][motion][key[0]][key[1]][key[2]]:
                        keyExist = 4

        if not keyExist:
            data[method][motion][key[0]]= {
                key[1]:{
                    key[2]:{
                        key[3]:{
                            "population": [self.popHist],
                            "objectives": [self.objHist]
                            # "violations": self.violHist
                        }
                    }
                }
            }

        else:
            if keyExist == 1:
                data[method][motion][key[0]][key[1]]= {
                    key[2]:{
                        key[3]:{
                            "population": [self.popHist],
                            "objectives": [self.objHist]
                            # "violations": self.violHist
                        }
                    }
                }
            elif keyExist == 2:
                data[method][motion][key[0]][key[1]][key[2]] = {
                    key[3]:{
                        "population": [self.popHist],
                        "objectives": [self.objHist]
                        # "violations": self.violHist
                    }
                }
            elif keyExist == 3:
                data[method][motion][key[0]][key[1]][key[2]][key[3]] = {
                    "population": [self.popHist],
                    "objectives": [self.objHist]
                    # "violations": self.violHist
                }
            elif keyExist == 4:
                results = data[method][motion][key[0]][key[1]][key[2]][key[3]]

                data[method][motion][key[0]][key[1]][key[2]][key[3]] = {
                    "population": results["population"] + [self.popHist],
                    "objectives": results["objectives"] + [self.objHist]
                    # "violations": results["violations"] + self.violHist
                }

        return data

    def storeFinalToBinary(self):
        method  = self.method
        motion  = self.motion
        key     = self.getKey()
        data    = self.loadBinary()

        data    = self.mergeData(data, method, motion, key)
        # print("================================================")
        self.writeToBinary(data, self.result_file)

    def storeFinalToJSON(self):
        method   = self.method
        motion   = self.motion
        key      = self.getKey()
        data     = self.loadJSON()

        data = self.mergeData(data, method, motion, key)
        self.writeToJson(data, self.result_file)
        # self.writeToJson(data, self.result_file)

    def compressToZip(self):

        # Try the deflated compression mode
        try:
            compression = zipfile.ZIP_DEFLATED
        except:
            compression = zipfile.ZIP_STORED

        # Compression modes
        modes = { zipfile.ZIP_DEFLATED: 'deflated', zipfile.ZIP_STORED: 'stored'}

        print('##  Creating compressed', self.zip_file ,'file...')
        zf = zipfile.ZipFile(self.zip_file, mode='w')
        try:
            print('##  The compression mode is:',  modes[compression])
            print('##  Adding files:'),
            print('##  \t- ',self.result_file)
            print('##  \t- ',self.result_info)
            zf.write(self.result_file, compress_type=compression)
            zf.write(self.result_info, compress_type=compression)
        finally:
            print('##  Closing the .zip file!')
            zf.close()
