import config as cg
import numpy as np

def decode_gait(ind):
    alpha = (cg.alpha_max-cg.alpha_min)*ind[0]/ind.Upp[0] + cg.alpha_min
    omega = (cg.omega_max-cg.omega_min)*ind[1]/ind.Upp[1] + cg.omega_min
    delta = (cg.delta_max-cg.delta_min)*ind[2]/ind.Upp[2] + cg.delta_min
    gait = [alpha, omega, delta]
    return gait

def decode_gait_no_omega(ind):
    alpha = (cg.alpha_max-cg.alpha_min)*ind[0]/ind.Upp[0] + cg.alpha_min
    delta = (cg.delta_max-cg.delta_min)*ind[1]/ind.Upp[1] + cg.delta_min
    gait = [alpha, delta]
    return gait

def decode_modified(ind):
    omega = (cg.omega_max-cg.omega_min)*ind[0]/ind.Upp[0] + cg.omega_min
    delta = (cg.delta_max-cg.delta_min)*ind[1]/ind.Upp[1] + cg.delta_min
    n_joints = len(ind)-2
    g_scale = [0]*n_joints
    for i in range(n_joints):
        g_scale[i] =  (cg.alpha_max-cg.alpha_min)*ind[2+i]/ind.Upp[2+i] + cg.alpha_min
    return [omega, delta] + g_scale

def decode_modified_no_omega(ind):
    delta = (cg.delta_max-cg.delta_min)*ind[0]/ind.Upp[0] + cg.delta_min
    n_joints = len(ind)-1
    g_scale = [0]*n_joints
    for i in range(n_joints):
        g_scale[i] =  (cg.alpha_max-cg.alpha_min)*ind[1+i]/ind.Upp[1+i] + cg.alpha_min
    return [delta] + g_scale

def decode_fourier(ind):
    omega = (cg.omega_max*ind[0])/(ind.Upp[0]) + cg.omega_min
    delta = (cg.delta_max*ind[1])/(ind.Upp[1]) + cg.delta_min
    first_genes =[omega, delta]
    # a_null = cg.a_null_max*(np.pi/180)*(2*ind[2]/ind.Upp[2]-1)
    # first_genes = [omega, delta, a_null]

    n_first = len(first_genes)
    n = len(ind) - n_first
    coeff = [0]*n

    for i in range(n):
        mean_bound = int(ind.Upp[i+n_first]/2)
        # if ind[i+1] < mean_bound:
            # coeff[i] = ((ind[i]-mean_bound)*cg.coeff_max*np.pi)/(mean_bound*180)
        if ind[i+n_first] > mean_bound+1:
            mean_bound += 1
        # coeff[i] = (cg.coeff_max*(ind[i+n_first]-mean_bound)*np.pi)/(mean_bound*180)
        coeff[i] = (cg.coeff_max*(ind[i+n_first]-mean_bound))/(mean_bound)

    return first_genes + coeff

def decode_fourier_no_omega(ind):
    delta = (cg.delta_max*ind[0])/(ind.Upp[0]) + cg.delta_min
    first_genes =[delta]

    n_first = len(first_genes)
    n = len(ind) - n_first
    coeff = [0]*n

    for i in range(n):
        mean_bound = int(ind.Upp[i+n_first]/2)
        if ind[i+n_first] > mean_bound+1:
            mean_bound += 1
        coeff[i] = (cg.coeff_max*(ind[i+n_first]-mean_bound))/(mean_bound)

    return first_genes + coeff

def decode_ind(motion, with_omega):
    if motion == "Modified":
        if with_omega:
            return decode_modified
        else:
            return decode_modified_no_omega
    elif motion == "FourierSeries":
        if with_omega:
            return decode_fourier
        else:
            return decode_fourier_no_omega
    elif motion == "MultiFourier":
        if with_omega:
            return decode_fourier
        else:
            return decode_fourier_no_omega
    else:
        if with_omega:
            return decode_gait
        else:
            return decode_gait_no_omega

if __name__ == "__main__":
    print("Empty function")
