# MOEA
from Algorithms.Utility.eval_objective_values import eval_objective_values
from Algorithms.Utility.print_elapsed_time import print_elapsed_time
from Algorithms.Utility.non_dominated_sort import non_dominated_sort
from Algorithms.Utility.penalty_function import penalty_function
from Algorithms.Utility import logData

# Configuration File
import config as cg

import time
import os

def plot_objectives(obj, eng, gen, method_name, motion):
    obj1 = [obj[i][0] for i in range(len(obj))]
    obj2 = [obj[i][1] for i in range(len(obj))]
    eng.plot_objectives(obj1, obj2, gen, method_name, motion, nargout=0)

def compute_elapsed_time(t1, sim_time, gen, elapsed_time, elapsed_computational_time, estimated_time):
    t2  = time.perf_counter()
    elapsed_time += t2 - t1
    elapsed_computational_time += t2 - t1 - sim_time
    estimated_time = print_elapsed_time(elapsed_time, (t2-t1), sim_time, elapsed_computational_time, \
                                                                    estimated_time, gen, cg.n_generations)
    return elapsed_time, elapsed_computational_time, estimated_time

def assign_violation_ranks(population):
    front =  non_dominated_sort(population)
    violated = penalty_function(population)
    if violated:
        for i in range(len(front)):
            print("==========VIOLATED===========")
            print("Front %d:" % (i+1))
            for j in range(len(front[i])):
                print("\trank: %d" % front[i][j].rank)

def store_data(gen, loggerObj, cwd, result_dir):
    if gen == cg.n_generations:
        os.chdir(result_dir)
        logData.final_save(loggerObj)
        print("## Storing the results!!!")
        os.chdir(cwd)
    # elif gen%5 == 0 and gen < (cg.n_generations-5):
    #     print("Temporary store generation: ", str(gen))
    #     logData.temporary_save(loggerObj)

def store_data_with_redundancy_check(gen, loggerObj, cwd, result_dir):
    # Plot objectives And Store History To File
    if gen == cg.n_generations:
        os.chdir(result_dir)
        print("=======================================================================\n")
        redundancy_check = True
        while redundancy_check:
            response = input("## Do you want to save these results?[Y/N]: ")
            if response.upper() == 'Y':
                logData.final_save(loggerObj)
                redundancy_check = False
                print("## Storing the results!!!")
                os.chdir(cwd)
            else:
                response = input("## Are you sure?[Y/N]: ")
                if response.upper() == 'Y':
                    redundancy_check = False
                else:
                    print("=======================================================================\n")

    elif gen%5 == 0 and gen < (cg.n_generations-5):
        print("Temporary store generation: ", str(gen))
        logData.temporary_save(loggerObj)

def main_interface(moea, method_name, motion, eng, toolbox, chrom_repr, L_BOUND, U_BOUND, loggerObj,
                                                                    result_dir, with_omega, redundancy_check):
    """ Main interface of (N+N) MOEA """
    # Get current working directory
    cwd = os.getcwd();

    # Initialize Time Parameters
    elapsed_time                = 0
    elapsed_computational_time  = 0
    estimated_time              = []
    t1 = time.perf_counter()

    # Evaluate Initial Population
    population, n_gen, sim_time = moea.init_population(loggerObj,toolbox,L_BOUND,U_BOUND,eng,motion,with_omega)

    # Log Data Of Generation 0 To History
    obj = [population[i].fitness.values for i in range(len(population))]
    if len(loggerObj.popHist) == 0:
        logData.save_history(loggerObj, population, obj)
        print("\nStore generation 0 to history\n" )
        # Plot objectives for generation 0
        plot_objectives(obj, eng, 0, method_name, motion)

    # Estimate Execution Time
    elapsed_time += time.perf_counter() - t1
    estimated_time = print_elapsed_time(elapsed_time, elapsed_time, sim_time, elapsed_computational_time, \
                                                                            estimated_time, 0, cg.n_generations)

    for gen in range(n_gen, cg.n_generations+1):
        t1 = time.perf_counter()
        print("=======================================================================")
        print('Generation: %d' % gen)
        print('Evaluate population')

        # Mating Selection & Breeding
        # offspring = moea.mating_selection(population, chrom_repr, toolbox, eng)
        offspring = moea.mating_selection(population, chrom_repr, toolbox, eng, loggerObj.cx, loggerObj.mx)

        # Evaluate Offsprings
        invalid_ind = [ind for ind in offspring if not ind.fitness.valid]
        sim_time = eval_objective_values(invalid_ind, eng, motion, with_omega)

        # Environmental Selection
        population = moea.environmental_selection(population + offspring, eng)

        # If Violations, Add Bad Fitness
        assign_violation_ranks(population)

        # Print Execution Time To Screen
        elapsed_time, elapsed_computational_time, estimated_time = compute_elapsed_time(t1, sim_time, gen, \
                                                        elapsed_time, elapsed_computational_time, estimated_time)
        # Log Data To History
        obj = [population[i].fitness.values for i in range(len(population))]
        logData.save_history(loggerObj, population, obj)
        plot_objectives(obj, eng, gen, method_name, motion)

        # Store Result Data
        if redundancy_check:
            store_data_with_redundancy_check(gen, loggerObj, cwd, result_dir)
        else:
            store_data(gen, loggerObj, cwd, result_dir)

    print("======================= End Of Optimization Run =======================")
    return elapsed_time

if __name__ == "__main__":
    main_interface()
    input("Press Enter to continue...")
