close all

tmp1=importdata('Data_Vc_030.txt');

%For data file Data_Vc_030
% Initial data point
n1=1;
% Final data point
n2=1085;
% Read time
time1=tmp1(n1:1:n2,1);

% Read head position
head_X=tmp1(n1:1:n2,2)/100;
head_Y=tmp1(n1:1:n2,3)/100;
head_Z=tmp1(n1:1:n2,4)/100;

% Read head orientation
head_angle_X=tmp1(n1:1:n2,5);
head_angle_Y=tmp1(n1:1:n2,6);
head_angle_Z=tmp1(n1:1:n2,7);

% Read 
CM_x=tmp1(n1:1:n2,8)/100;
CM_y=tmp1(n1:1:n2,9)/100;

% Read Orientation
orientation=tmp1(n1:1:n2,10);

% Read Orientation Reference
orientation_ref=tmp1(n1:1:n2,11);

% Read offset
phi_o=tmp1(n1:1:n2,12);

% Read Forward speed
speed1=tmp1(n1:1:n2,13)/100;

label_font_size = 14;
figpos = [972   483   603   387]; % Avhandling


% Plot Center of Mass position X
window = figure(1);
set(window, 'color', 'white');
hold on
plot(t,p_CM(:,1),'linewidth', 2)
plot(time1,CM_x,'k','linewidth', 2)
hold off
grid on
xlabel('Time [sec]','FontSize',label_font_size);
ylabel('p_x [m]','FontSize',label_font_size);
set(gcf,'Position',figpos)

% Plot Center of Mass position Y
window = figure(2);
set(window, 'color', 'white');
hold on
plot(t,p_CM(:,2),'linewidth', 2)
plot(time1,CM_y,'k','linewidth', 2)
hold off
grid on
xlabel('Time [sec]','FontSize',label_font_size);
ylabel('p_y [m]','FontSize',label_font_size);
set(gcf,'Position',figpos)

% Plot Path X-Y

window = figure(3);
set(window, 'color', 'white');
plot(p_CM(:,1), p_CM(:,2), 'linewidth', 2)
hold on
plot([p_CM(1,1) p_CM(end,1)], [0 0], 'r--','linewidth', 2)
plot(CM_x, CM_y,'k', 'linewidth', 2)
grid on
hold off
xlabel('p_x [m]','FontSize',label_font_size);
ylabel('p_y [m]','FontSize',label_font_size);
ylim([-2 1])
h = legend('Path of CM of robot', 'Reference path','Simulation path', 'location', 'northeast');
set(h,'fontsize',14,'interpreter','latex');
set(gcf,'Position',figpos)
legend('boxoff')


% Plot Orientation
window = figure(4);
set(window, 'color', 'white');
plot(t_sim, psi_path_sim*180/pi,'linewidth', 2)
hold on
plot(t_sim, psi_d*180/pi,'r--', 'linewidth', 2)
plot(time1,orientation,'k','linewidth', 2)
grid on
hold off
xlabel('Time [sec]','FontSize',label_font_size);
ylabel('$\overline{\theta}$ [deg]','FontSize',label_font_size,'interpreter','latex')
h = legend('$\overline{\theta}$', '$\overline{\theta}_{\mathbf{ref}}$', 'location', 'northeast');
set(h,'fontsize',14,'interpreter','latex');
set(gcf,'Position',figpos)
legend('boxoff')

% Plot Forward Velocity
window = figure(5);
set(window, 'color', 'white');
hold on
plot(t,speed,'linewidth', 2)
plot(time1,speed1,'k','linewidth', 2)
hold off
grid on
xlabel('Time [sec]','FontSize',label_font_size);
ylabel('$\overline{v}_t$ [m/s]','FontSize',16,'interpreter','latex');
set(gcf,'Position',figpos)

% Plot Offset
window = figure(6);
set(window, 'color', 'white');
hold on
plot(t_sim,phi_o_commanded_sim*180/pi,'linewidth', 2)
plot(time1,phi_o,'k','linewidth', 2)
hold off
grid on
xlabel('Time [sec]','FontSize',label_font_size);
ylabel('$\phi_0$ [deg]','FontSize',label_font_size,'interpreter','latex')
set(gcf,'Position',figpos)
