global n A D sin_theta_diag cos_theta_diag

%previewfig(gcf,'bounds','tight','color','rgb');
%exportfig(gcf,'speed_comparison.eps','bounds','tight','color','rgb');

% Start time for plotting.
t_start = 5;

% Determines the start index.
start_index = 1;
while t(start_index, 1) < t_start
    start_index = start_index + 1;
end

% Stop time for plotting.
t_stop = 79.9;

% Calculates the link positions.
if 1
    x_sim = zeros(size(t, 1), n);
    y_sim = zeros(size(t, 1), n);
    
    for i = 1:size(t, 1)
        % Calculates diagonal matrices that are used in the proceeding calculations
        sin_theta_diag = calculate_sin_theta_diag(theta(i, :)');
        cos_theta_diag = calculate_cos_theta_diag(theta(i, :)');
        
        % Calculates link positions.
        z = calculate_z(theta(i, :)', [], p_CM(i, :)');
        
        % Extracts the x and y coordinates
        x_sim(i, :) = z(1:n, 1)';
        y_sim(i, :) = z(n+1:2*n, 1)';
    end
end

% Plots the trace of the head of the snake robot.
if 1
    
    % Sets figure properties
    %window = figure(10);
    %clf;
    %set(window, 'color', 'white');
    subplot(2, 1, 1);
    set(gcf, 'color', 'white');
    hold on;
    %grid on;
    
    % The link color.
    %link_color = 'bo-';
    link_color = 'b';
    
    next_index = start_index;
    % Draws the initial shape and position of the snake robot.
    drawSnakeRobot_plotpurposes(theta(next_index, :)', p_CM(next_index, :)', link_color);
    
    next_index = start_index;
    while t(next_index) < t_stop
        next_index = next_index + 1;
    end
    % Draws the final shape and position of the snake robot.
    drawSnakeRobot_plotpurposes(theta(next_index, :)', p_CM(next_index, :)', link_color);
    
    % Plots the trace of the head.
    plot(x_sim(start_index:next_index, n), y_sim(start_index:next_index, n), 'k:')
    
    axis('equal');
    xlabel('X [m]','FontSize',14);
    ylabel('Y [m]','FontSize',14)
    set(gca,'FontSize',14)
    
    hold off
    %axis tight
    %axis([-0.5, 3, -0.3, 0.3]);
    %set(gcf, 'position', [624 497 684 201])
end

% Indicates on the head trace when turning motion is started/stopped.
if 1
    hold on
    
    % Indicates where turning is started and stopped.
    % Determines the start index.
    turning_start_index = 1;
    while t(turning_start_index, 1) < 20
        turning_start_index = turning_start_index + 1;
    end
    turning_stop_index = 1;
    while t(turning_stop_index, 1) < 30
        turning_stop_index = turning_stop_index + 1;
    end
    plot(x_sim(turning_start_index, n), y_sim(turning_start_index, n), '^r')
    plot(x_sim(turning_stop_index, n), y_sim(turning_stop_index, n), 'vr')
    
    % Indicates where turning is started and stopped.
    % Determines the start index.
    turning_start_index = 1;
    while t(turning_start_index, 1) < 50
        turning_start_index = turning_start_index + 1;
    end
    turning_stop_index = 1;
    while t(turning_stop_index, 1) < 60
        turning_stop_index = turning_stop_index + 1;
    end
    plot(x_sim(turning_start_index, n), y_sim(turning_start_index, n), '^r')
    plot(x_sim(turning_stop_index, n), y_sim(turning_stop_index, n), 'vr')

    hold off
end

% Plots the difference in x coordinate of two neighbouring links.
if 1
    next_index = start_index;
    while t(next_index) < t_stop
        next_index = next_index + 1;
    end
    
    delta_x = x_sim(:, 5) - x_sim(:, 4);
    
    %window = figure(15);
    %set(window, 'color', 'white');
    subplot(3, 1, 2);
    plot(t(start_index:next_index, 1)-t_start, delta_x(start_index:next_index, 1), 'b')
    grid on
    hold on
    xlabel('Time [s]','FontSize',14);
    ylabel('x_5 - x_4 [m]','FontSize',14);
    set(gca,'FontSize',14)
    hold off
    axis([t(start_index, 1)-t_start t(next_index, 1)-t_start -0.2 0.2])
    %set(gcf, 'position', [624 497 684 201])
end

% Plots the difference in y coordinate of two neighbouring links.
if 1
    next_index = start_index;
    while t(next_index) < t_stop
        next_index = next_index + 1;
    end
    
    delta_y = y_sim(:, 5) - y_sim(:, 4);
    
    %window = figure(16);
    %set(window, 'color', 'white');
    subplot(3, 1, 3);
    plot(t(start_index:next_index, 1)-t_start, delta_y(start_index:next_index, 1), 'b')
    grid on
    hold on
    xlabel('Time [s]','FontSize',14);
    ylabel('y_5 - y_4 [m]','FontSize',14);
    set(gca,'FontSize',14)
    hold off
    axis([t(start_index, 1)-t_start t(next_index, 1)-t_start -0.2 0.2])
    %set(gcf, 'position', [624 497 684 201])
end

% Plots the CM velocity in the global x direction.
if 1
    next_index = start_index;
    while t_sim(next_index) < t_stop
        next_index = next_index + 1;
    end

    window = figure(20);
    set(window, 'color', 'white');
    plot(t_sim(start_index:next_index, 1), p_CM_dot_sim(start_index:next_index, 1), 'b')
    grid on
    hold on
    xlabel('Time [s]');
    ylabel('$\dot{p_x}$ [m/s]','interpreter','latex')
    hold off
    axis tight
    set(gcf, 'position', [624 497 684 201])
end

% Plots the joint torques at joint 5.
if 1
    next_index = 1;
    while t_sim(next_index) < t_stop
        next_index = next_index + 1;
    end
    
    window = figure(40);
    set(window, 'color', 'white');
    plot(t_sim(1:next_index, 1), u_sim(1:next_index, 5), 'b')
    axis([t_sim(1, 1) t_sim(next_index, 1) -15 15]);
    grid on
    xlabel('Time [s]');
    ylabel('$u_5$ [Nm]','interpreter','latex')
    %set(gcf, 'position', [624 497 684 201])
end

% Plots the average of the joint angles.
if 1
    phi_av = [];
    for i = 1:size(phi, 1)
        phi_av(end+1, 1) = sum(phi(i, 1:n-1))/(n-1);
    end
    %window = figure(50);
    %set(window, 'color', 'white');
    subplot(2, 1, 2);
    plot(t(1:end, 1), -1*phi_av(1:end, 1)*180/pi, 'b')
    grid on
    xlabel('Time [s]','FontSize',14)
    %ylabel('\phi_{i} [deg]')
    ylabel('$\overline{\phi}$ [deg]','interpreter','latex','FontSize',14)
    set(gca,'FontSize',14)
end

% Plots optimal delta as function of number of links, n.
if 1
    n_vector = (3:30)';
    delta_vector = zeros(size(n_vector, 1), 1);
    for n_counter = 1:size(n_vector, 1)
        % Sets number of links.
        n = n_vector(n_counter, 1);
        % Initializes model parameters.
        initModelParameters;
        
        % Calculates k_delta.
        AD_bar = round(n*A*D'*inv(D*D'))/n;
        delta_e = (0:0.01:pi/2)';
        kdelta_e = [];
        for k = 1:size(delta_e, 1)
            k_delta_next = 0;
            for i = 1:n-1
                for j = 1:n-1
                    k_delta_next = k_delta_next + AD_bar(i, j)*sin((j-i)*delta_e(k, 1));
                end
            end
            kdelta_e(end+1, 1) = k_delta_next;
        end
        
        % Finds maximum value of k_delta.
        [k_delta_max, max_index_delta] = max(kdelta_e);
        % In case several max values were found.
        k_delta_max = k_delta_max(1, 1);
        max_index_delta = max_index_delta(1, 1);
        % Optimal delta.
        delta_vector(n_counter, 1) = delta_e(max_index_delta, 1)*180/pi;
    end
    
    % Plots optimal delta.
    window = figure(95);
    set(window, 'color', 'white');
    plot(n_vector, delta_vector)
    grid on
    xlabel('Number of links, $N$','interpreter','latex','FontSize',12)
    ylabel('Optimal ${\delta}$ [deg]','interpreter','latex','FontSize',12)
    set(gca,'FontSize',10)
end
    
if 1
    % Creates a combined plot from individual figures.
    window = figs2subplots([10 50], [2 1]);
    set(window, 'color', 'white');
end

%exportfig(gcf,'relative_link_motion.eps','bounds','tight','color','rgb');
%printeps(95, 'optimal_delta_of_N') % Replaces Helvetica font in eps-file.
