function [theta_pathframe, p_pathframe] = calculate_pathframe_state(t, theta, p)
% Converts the global frame link angles and position of the snake robot
% into the corresponding angles and position in the current path frame.
% The path frame: The frame with origo in the previous waypoint and
% x axis pointing towards the next waypoint.

global waypoints r_acceptance cur_WP_index cur_path_angle p_WP_pathframe ...
    R_globalframe_pathframe_tr WP_switch_times

% The link angles with respect to the x axis of the path frame.
theta_pathframe = theta - cur_path_angle;

% The vector from the path frame to the CM of the snake described in
% the global frame.
p_pathframe_global = p - waypoints(cur_WP_index-1, :)';

% The position of the snake robot wrt the path frame.
p_pathframe = R_globalframe_pathframe_tr*p_pathframe_global;

% Determines if the current waypoint has been reached.
activate_next_WP = 0;

% Checks if the snake has reached the acceptance limit.
if p_pathframe(1, 1) >= p_WP_pathframe(1, 1)
    activate_next_WP = 1;
else
    % The distance from the snake to the waypoint along the x axis of the path frame.
    WP_dist_x = abs(p_WP_pathframe(1, 1) - p_pathframe(1, 1));
    % The distance from the snake to the waypoint along the y axis of the path frame.
    WP_dist_y = abs(p_WP_pathframe(2, 1) - p_pathframe(2, 1));
    % Calculates the exact distance if the CM of the snake is inside a
    % rectangular area around the waypoint.
    if WP_dist_x <= r_acceptance && WP_dist_y <= r_acceptance
        distance = sqrt(WP_dist_x^2 + WP_dist_y^2);
        % Checks if CM is inside acceptance circle.
        if distance <= r_acceptance
            activate_next_WP = 1;
        end
    end
end

% Activates the next waypoint.
if activate_next_WP && cur_WP_index < size(waypoints, 1)
    
    % Increases the WP index.
    cur_WP_index = cur_WP_index + 1;
    
    % Stores the switch time.
    WP_switch_times(end+1, 1) = t;
    
    % Calculates the angle of the straight path between the previous and
    % the current waypoint.
    x_prev = waypoints(cur_WP_index-1, 1);
    y_prev = waypoints(cur_WP_index-1, 2);
    x_cur = waypoints(cur_WP_index, 1);
    y_cur = waypoints(cur_WP_index, 2);
    cur_path_angle = atan2(y_cur-y_prev, x_cur-x_prev);
    
    % Sinus and cosinus values.
    sin_path_angle = sin(cur_path_angle);
    cos_path_angle = cos(cur_path_angle);
    
    % Rotation matrix from the path frame to the global frame.
    % The path frame is the frame with origo in the previous waypoint and
    % x axis pointing towards the next waypoint.
    R_globalframe_pathframe_tr = [cos_path_angle sin_path_angle ; -sin_path_angle cos_path_angle];
    
    % The vector from the path frame to the current waypoint described in
    % the global frame.
    p_WP_global = waypoints(cur_WP_index, :)' - waypoints(cur_WP_index-1, :)';
    
    % The position of the current waypoint in the path frame.
    p_WP_pathframe = R_globalframe_pathframe_tr*p_WP_global;
    
end
