function [phi_dot_dot, theta_dot_dot, p_CM_dot_dot] = calculate_q_dot_dot_CM_USR(t, theta_dot, p_CM_dot, fx, fy, u,...
                        n, J, D, e, V_x, V_y, V, V_1, V_2, sin_theta_diag,... 
                        cos_theta_diag, mi_n, lambda1, lambda2, lambda3, m_tot, K)
%
% Calculates and returns the acceleration of theta and p_CM and the constraint joint forces
% for each link of the snake robot. Theta are the absolute link angles and
% p_CM is the position of the center of mass of the snake.
%

%Calculate useful matrixes

Mtot=[m_tot+e'*mi_n*sin_theta_diag*sin_theta_diag*e,...
    -e'*mi_n*sin_theta_diag*cos_theta_diag*e;...
    -e'*mi_n*sin_theta_diag*cos_theta_diag*e,...
    m_tot+e'*mi_n*cos_theta_diag*cos_theta_diag*e;];

% Calculate the inverse of the matrix
Mtot_inv=Mtot\eye(2);


M11=Mtot_inv(1,1); M12=Mtot_inv(1,2); M21=Mtot_inv(2,1); M22=Mtot_inv(2,2);

A_1=-sin_theta_diag*sin_theta_diag*mi_n*K'*sin_theta_diag...
    -sin_theta_diag*cos_theta_diag*mi_n*K'*cos_theta_diag;
A_2=-sin_theta_diag*sin_theta_diag*mi_n*K'*cos_theta_diag...
    +sin_theta_diag*cos_theta_diag*mi_n*K'*sin_theta_diag;
A_3=-sin_theta_diag*cos_theta_diag*mi_n*K'*sin_theta_diag...
    +sin_theta_diag*sin_theta_diag*mi_n*K'*cos_theta_diag;
A_4=sin_theta_diag*cos_theta_diag*mi_n*K'*sin_theta_diag...
    +cos_theta_diag*cos_theta_diag*mi_n*K'*cos_theta_diag;
A_5=sin_theta_diag*cos_theta_diag*mi_n*K'*cos_theta_diag...
    -cos_theta_diag*cos_theta_diag*mi_n*K'*sin_theta_diag;
A_6=cos_theta_diag*cos_theta_diag*mi_n*K'*sin_theta_diag...
    -sin_theta_diag*cos_theta_diag*mi_n*K'*cos_theta_diag;

K_1=M11*e'*sin_theta_diag*sin_theta_diag*mi_n...
    -M12*e'*sin_theta_diag*cos_theta_diag*mi_n;
K_2=-M11*e'*sin_theta_diag*cos_theta_diag*mi_n...
    +M12*e'*cos_theta_diag*cos_theta_diag*mi_n;
K_3=M21*e'*sin_theta_diag*sin_theta_diag*mi_n...
    -M22*e'*sin_theta_diag*cos_theta_diag*mi_n;
K_4=M21*e'*sin_theta_diag*cos_theta_diag*mi_n...
    -M22*e'*cos_theta_diag*cos_theta_diag*mi_n;
K_5=-sin_theta_diag*V_2-sin_theta_diag*V*sin_theta_diag*sin_theta_diag*mi_n*e...
    -cos_theta_diag*V*sin_theta_diag*cos_theta_diag*mi_n*e;
K_6=cos_theta_diag*V_2+sin_theta_diag*V*sin_theta_diag*cos_theta_diag*mi_n*e...
    +cos_theta_diag*V*cos_theta_diag*cos_theta_diag*mi_n*e;

% Matrices for the USR

% Calculates the inertia matrix
M_theta=J+sin_theta_diag*V_1*sin_theta_diag+cos_theta_diag*V_1*cos_theta_diag...
    -sin_theta_diag*V*A_1+cos_theta_diag*V*A_4+K_5*K_1*K'*sin_theta_diag...
    -K_5*K_2*K'*cos_theta_diag+K_6*K_3*K'*sin_theta_diag...
    +K_6*K_4*K'*cos_theta_diag+lambda1;

% Calculates various other matrices

W_theta=-sin_theta_diag*V_1*cos_theta_diag+cos_theta_diag*V_1*sin_theta_diag...
    -sin_theta_diag*V*A_2+cos_theta_diag*V*A_5+K_5*K_1*K'*cos_theta_diag...
    +K_5*K_2*K'*sin_theta_diag+K_6*K_3*K'*cos_theta_diag-K_6*K_4*K'*sin_theta_diag;

V_theta=-sin_theta_diag*V*diag(theta_dot)*A_3+cos_theta_diag*V*diag(theta_dot)*A_6...
    -K_5*K_2*diag(theta_dot)*K'*sin_theta_diag-K_5*K_1*diag(theta_dot)*K'*cos_theta_diag...
    +K_6*K_4*diag(theta_dot)*K'*sin_theta_diag-K_6*K_3*diag(theta_dot)*K'*cos_theta_diag...
    +lambda2+lambda3*diag(abs(theta_dot));

N_theta=(sin_theta_diag*V*sin_theta_diag*cos_theta_diag*mi_n...
    +cos_theta_diag*V*cos_theta_diag*cos_theta_diag*mi_n-K_5*K_2+K_6*K_4)*diag(theta_dot);

P_theta=(sin_theta_diag*V*sin_theta_diag*sin_theta_diag*mi_n...
    +cos_theta_diag*V*sin_theta_diag*cos_theta_diag*mi_n+K_5*K_1+K_6*K_3)*diag(theta_dot);

Kx=-sin_theta_diag*V-K_5*M11*e'-K_6*M21*e';

Ky=cos_theta_diag*V-K_5*M12*e'-K_6*M22*e';
    

% Matrices for the USR including the foil model
M_theta_t=M_theta;

W_theta_t=W_theta;

V_theta_t=V_theta;

N_theta_t=N_theta;

P_theta_t=P_theta;

Kxt=Kx;
Kyt=Ky;

% Calculates a vector of squared angular link velocities
theta_dot_sq = zeros(n, 1);
theta_dot_abs = zeros(n, 1);

for i = 1:n
    theta_dot_sq(i, 1) = theta_dot(i, 1)*theta_dot(i, 1);
    theta_dot_abs(i, 1) = theta_dot(i, 1)*abs(theta_dot(i, 1));
end

% Constructs the inertia matrix of the system
M_inv=M_theta_t\eye(n);

% Calculates a matrix related to Coriolis and centripetal forces
W_th=W_theta_t*theta_dot_sq+V_theta_t*theta_dot+N_theta_t*(e*p_CM_dot(1, 1)-V_x)...
    +P_theta_t*(e*p_CM_dot(2, 1)-V_y)+Kxt*(fx)+Kyt*(fy);

% Calculates the theta_dot_dot
theta_dot_dot =M_inv*(D'*u-W_th);

px_dot_dot=(-K_1*K'*sin_theta_diag+K_2*K'*cos_theta_diag)*theta_dot_dot...
    +(-K_1*K'*cos_theta_diag-K_2*K'*sin_theta_diag)*theta_dot_sq...
    +(K_2*diag(theta_dot)*K'*sin_theta_diag+K_1*diag(theta_dot)*K'*cos_theta_diag)*theta_dot...
    +K_2*diag(theta_dot)*(e*p_CM_dot(1, 1)-V_x)-K_1*diag(theta_dot)*(e*p_CM_dot(2, 1)-V_y)...
    +M11*e'*(fx)+M12*e'*(fy);
py_dot_dot=(-K_3*K'*sin_theta_diag-K_4*K'*cos_theta_diag)*theta_dot_dot...
    +(-K_3*K'*cos_theta_diag+K_4*K'*sin_theta_diag)*theta_dot_sq...
    +(-K_4*diag(theta_dot)*K'*sin_theta_diag+K_3*diag(theta_dot)*K'*cos_theta_diag)*theta_dot...
    -K_4*diag(theta_dot)*(e*p_CM_dot(1, 1)-V_x)-K_3*diag(theta_dot)*(e*p_CM_dot(2, 1)-V_y)...
    +M21*e'*(fx)+M22*e'*(fy);

% px_dot_dot=K1*theta_dot_dot+K2*theta_dot_sq+K3*theta_dot+M11*e'*(fx+fxt)+M12*e'*(fy+fyt)...
%     -e'*(M11*S_beta_psi*ro*pi*B.^3*a+M12*C_beta_psi*ro*pi*B.^3*a)*theta_dot_dot;
% py_dot_dot=K4*theta_dot_dot+K5*theta_dot_sq+K6*theta_dot+M21*e'*(fx+fxt)+M22*e'*(fy+fyt)...
%     -e'*(M21*S_beta_psi*ro*pi*B.^3*a+M22*C_beta_psi*ro*pi*B.^3*a)*theta_dot_dot;

p_CM_dot_dot=[px_dot_dot;py_dot_dot];

%teller=[teller;M(1)];
% t_save=[t_save;t];
%zfsd=-lambda1*theta_dot_dot-lambda2*theta_dot-lambda3*theta_dot_abs
% Calculates the joint angle acceleration
phi_dot_dot=D*theta_dot_dot;