function fig = drawSnakeRobot(theta, p, snake_color)
%
% Draws the snake robot in a graph window.
%

global n l sin_theta_diag cos_theta_diag e K...
    visualizeCMposition visualizeWaypoints waypoints v_x v_y

% disp(p)

% Calculates the position of the links.
sin_theta_diag = calculate_sin_theta_diag(theta,n);
cos_theta_diag = calculate_cos_theta_diag(theta,n);

z = calculate_z(p, e, K, sin_theta_diag, cos_theta_diag);
% Partitions the position vector.
x = z(1:n);
y = z(n+1:2*n);

% Draws the snake robot
for j = 1:n
    % Calculates the global frame coordinates of the beginning of the next link.
    startx = x(j, 1) - l*cos(theta(j, 1));
    starty = y(j, 1) - l*sin(theta(j, 1));
    % Calculates the global frame coordinates of the end of the next link.
    endx = x(j, 1) + l*cos(theta(j, 1));
    endy = y(j, 1) + l*sin(theta(j, 1));
    
    if 1
        % Draws the next link as a line.
        %plot([startx ; endx], [starty ; endy], 'bo-', 'linewidth', 1.3)
        %plot([startx ; endx], [starty ; endy], 'ko-', 'linewidth', 1.3);
        fig = plot([startx ; endx], [starty ; endy], [snake_color,'o-'], 'linewidth', 1.3);
        % Draws a point at the CM of each link.
        %plot(x, y, 'ro')
        %axis on
        %grid on
    end
end

% Draws the position of the CM (center of mass)
if visualizeCMposition
    plot(p(1, 1), p(2, 1), 'r*')
end

% Draws the waypoints.
if visualizeWaypoints
    plot(waypoints(:, 1), waypoints(:, 2), 'ks:')
    %plot(waypoints(cur_WP_index, 1), waypoints(cur_WP_index, 2), 'rs')
end

% Close-up rectangular view
%axis([p(1,1)-n*l-0.3, p(1,1)+n*l+0.3, p(2,1)-n*l-0.1, p(2,1)+n*l+0.1]);
% Close-up flat view
%axis([p(1,1)-n*l-0.1, p(1,1)+n*l+0.1, p(2,1)-6*l, p(2,1)+6*l]);
% Zoomed out view
%axis([-0.7, 3.5, -0.4, 0.4]);
% Zoomed out view
%axis([-0.7, 1.5, -0.4, 0.4]);
% 
%axis([p(1,1)-n*l-1.5, p(1,1)+n*l+1.5, -1.8, 1.8]);
%axis([-0.5, 20, -0.8, 20]);

% Close-up rectangular view
%axis([p(1,1)-n*l-0.2, p(1,1)+n*l+0.2, p(2,1)-n*l-0.2, p(2,1)+n*l+0.2]);

% draw the current
ax_x = p(1,1)-n*l-1.5:0.5:p(1,1)+n*l+1.5;
ax_y = p(2,1)-n*l-1.5:0.5:p(2,1)+n*l+1.5;
[ax_x, ax_y] = meshgrid(ax_x, ax_y);

quiver(ax_x,ax_y,v_x*ones(size(ax_x)),v_y*ones(size(ax_y)),0,'b')

