function drawSnakeRobot(t, theta, p)
%
% Draws the snake robot in a graph window.
%

global n l fignumber sin_theta_diag cos_theta_diag ...
    visualizeCMposition visualizeWaypoints waypoints

% Sets figure properties
window = figure(fignumber);

% Indicates that the erasing and redrawing of a rendering occurs off-screen
% to avoid flickering
set(window, 'DoubleBuffer', 'on');
clf

% Sets the background color.
set(window, 'color', 'white');
hold on;
%grid on;

axis('equal');
xlabel('X [m]');
ylabel('Y [m]');
% Rounds the time to two decimals precision
%t = round(t*100)/100;
title(['t = ' num2str(t)]);

% Calculates the position of the links.
sin_theta_diag = calculate_sin_theta_diag(theta);
cos_theta_diag = calculate_cos_theta_diag(theta);
z = calculate_z(theta, [], p);
% Partitions the position vector.
x = z(1:n);
y = z(n+1:2*n);

% Draws the snake robot
for j = 1:n
    % Calculates the global frame coordinates of the beginning of the next link.
    startx = x(j, 1) - l*cos(theta(j, 1));
    starty = y(j, 1) - l*sin(theta(j, 1));
    % Calculates the global frame coordinates of the end of the next link.
    endx = x(j, 1) + l*cos(theta(j, 1));
    endy = y(j, 1) + l*sin(theta(j, 1));
    
    % The color of the link to be drawn
    %link_color = 'b.-';
    
    %link_color = 'bo-';
    %link_color = 'ro--';
    
    if 0
        if (t >= 10 && t < 20) || t >= 25
            % Calculates the global frame coordinates of the beginning of the next link.
            startx = x(j, 1) - l;
            starty = y(j, 1);
            % Calculates the global frame coordinates of the end of the next link.
            endx = x(j, 1) + l;
            endy = y(j, 1);
        end
    end
    
    if 1
        % Draws the next link as a line.
        %plot([startx ; endx], [starty ; endy], 'bo-', 'linewidth', 1.3)
        plot([startx ; endx], [starty ; endy], 'b-', 'linewidth', 1.3)
        % Draws a point at the CM of each link.
        %plot(x, y, 'ro')
        %axis on
        %grid on
    end
    
    if 0
        if t >= 0 && t < 5
            % Draws the next link as a line.
            plot([startx ; endx], [starty ; endy], 'b-', 'linewidth', 1.3)
            % Draws a point at the CM of each link.
            plot(x, y, 'ro', 'linewidth', 1.3)
            axis on
            grid on
        elseif t >= 5 && t < 15
            % Draws a point at the CM of each link.
            plot(x, y, 'ro', 'linewidth', 1.3)
            axis off
            grid off
        elseif t >= 15 && t < 20
            % Draws the next link as a line.
            plot([startx ; endx], [starty ; endy], 'b-', 'linewidth', 1.3)
            % Draws a point at the CM of each link.
            plot(x, y, 'ro', 'linewidth', 1.3)
            axis on
            grid on
        elseif t >= 20 && t < 30
            % Draws a point at the CM of each link.
            plot(x, y, 'ro', 'linewidth', 1.3)
            axis off
            grid off
        else
            % Draws the next link as a line.
            plot([startx ; endx], [starty ; endy], 'b-', 'linewidth', 1.3)
            % Draws a point at the CM of each link.
            plot(x, y, 'ro', 'linewidth', 1.3)
            axis on
            grid on
        end
    end
    
end

% Draws the position of the CM (center of mass)
if visualizeCMposition
    plot(p(1, 1), p(2, 1), 'r*')
end

% Draws the waypoints.
if visualizeWaypoints
    plot(waypoints(:, 1), waypoints(:, 2), 'ks:')
    %plot(waypoints(cur_WP_index, 1), waypoints(cur_WP_index, 2), 'rs')
end

% Close-up rectangular view
%axis([p(1,1)-n*l-0.3, p(1,1)+n*l+0.3, p(2,1)-n*l-0.1, p(2,1)+n*l+0.1]);
% Close-up flat view
%axis([p(1,1)-n*l-0.1, p(1,1)+n*l+0.1, p(2,1)-6*l, p(2,1)+6*l]);
% Zoomed out view
%axis([-0.7, 3.5, -0.4, 0.4]);
% Zoomed out view
%axis([-0.7, 1.5, -0.4, 0.4]);

axis([p(1,1)-n*l-1.5, p(1,1)+n*l+1.5, -0.8, 1.8]);
%axis([-2, 20, -1, 2]);

hold off
