function [pop, decoded_pop] = load_result(algorithm, motion, pop_size, gen_size, crossover, mutation, directory)

%     clear all

    addpath(pwd)
    return_path = pwd;
    cd(directory);

    file = [ algorithm, '_', motion, '_', pop_size, '_', gen_size, '_',...
                  crossover,'_', mutation];

    %% Read the data file
%     data_nr = 1;
%     [population, objectives] = read_file(file, data_nr, 'end');
    [population, objectives] = read_file(file, 'all'); 

    %% Sort Objectives
    decoded_pop = {};
    for k = 1:numel(population)
        [sorted_obj, indexes] = sort_objectives(objectives{k}, 1);
        population{k} = double(population{k}(indexes,:));
        objectives{k} = double(objectives{k}(indexes,:));
        decoded_pop{k} = decode_data(population{k}, motion);
    end
%     [sorted_obj, indexes] = sort_objectives(objectives, 1);
%     population = population(indexes,:);
%     objectives = objectives(indexes,:);
    
    %% Decode the population
    
    %% Get the objective values
%     [P_avg, v_avg] = get_objectives(objectives);
    
    %% Save population and objectives to .mat file (Used for further analysis)
%     obj = [objectives(:,1),-objectives(:,2)];
%     omega_max = 210;
    % save_file(file, data_nr, decoded_pop, obj, omega_max, motion)
    
    pop = population;
    cd(return_path);
    

    %% ========================= Utility Functions ============================
    function [population, objectives] =  read_file(file_read, gen_nr)

        if exist([file_read,'.mat'],'file') == 2
            data = load(file_read);      
        else
            cd(return_path);
            error('############### Invalid file ####################')
        end
        
        fprintf('\n');
        disp('==================================');
        disp('## Select data: ');
        for i = numel(data.population)
            output_text = sprintf('##\t- Data%d',i);
            disp(output_text);
        end
        
        disp(numel(data.population))
        
        data_nr = 0;
        while isempty(data_nr) || ((data_nr > numel(data.population)) || (data_nr < 1)) 
            text_input = sprintf('Select a nr. between 1-%d: ', numel(data.population));
            data_nr = int16(input(text_input)); 
        end        
%         data_nr = int16(data_nr);
        
        if strcmpi(gen_nr,'all')
            population = data.population{data_nr};
            objectives = data.objectives{data_nr};
        elseif strcmpi(gen_nr,'end')
            population = double(data.population{data_nr}{end});
            objectives = double(data.objectives{data_nr}{end});
        else
            if str2nr(gen_nr) <= numel(data.population{data_nr})
                population = double(data.population{data_nr}{str2nr(gen_nr)});
                objectives = double(data.objectives{data_nr}{str2nr(gen_nr)});
            else
                cd(return_path);
                error('Invalid generation')
            end
        end
    end

    function save_file(file, data_nr, pop, obj, omega_opt, motion)

        population  = pop;
        X_1         = char([]);
        Y_1         = char([]);

        objectives  = obj;
        X_2         = char([]);
        Y_2         = char([]);

        velocity    = [0,0.2,0.4,0.6,0.8,1];
        range       = [];

        % ============ Remove Outliers =============
        omega_opt = 210;
        i = find(population(:,1) == omega_opt);
        population = population(i,:);
        objectives = objectives(i,:);

        str_len = 5;
        for i = 1:size(population,1)
            a = sprintf('i%d',i);
            for j =1:str_len-numel(a)
                a = [a,' '];
            end
            X_1(i,:) = a;     
        end

        if strcmpi(motion,'Lateral')
            Y_1 = ['alpha'; 'omega'; 'delta'];
            Y_2 = ['Avgerage Power  '; 'Average Velocity'];
        elseif strcmpi(motion,'Modified')
            Y_1 = ['alpha'; 'omega'; 'delta'; 'g1   ';'g2   ';'g3   ';'g4   ';'g5   ';'g6   ';'g7   ';'g8   ';'g9   '];
            Y_2 = ['Avgerage Power  '; 'Average Velocity'];
        elseif strcmpi(motion,'FourierSeries')
            Y_1 = ['omega';'delta';];
            str_len = numel(Y_1(1,:));
            n_coeff = numel(population(1,3:end))/2;
            a_coeff = [];
            b_coeff = [];
            for i = 1:n_coeff
                str1 = sprintf('a%d',i);
                str2 = sprintf('b%d',i);
                % Add spacing
                for j = 1:str_len-numel(str1)
                    str1 = [str1, ' '];
                    str2 = [str2, ' ' ];
                end
                a_coeff = [a_coeff; str1];
                b_coeff = [b_coeff; str2];
            end
            Y_1 = [Y_1;a_coeff;b_coeff];
            Y_2 = ['Avgerage Power  '; 'Average Velocity'];
        else
            error('No variable naming done yet!!!!!')
            cd(return_path);
        end  

    %     disp(population(:,4:5));

        for i = 1:numel(velocity)-1
            range{end+1} = find(objectives(:,2) >= velocity(i) & objectives(:,2) <= velocity(i+1));
        end

        for i = 1:numel(range)
           disp(['Range',num2str(i),': ']);
           disp(range{i}(end))
        end

        save_pop = [num2str(data_nr),'_',file,'_population.mat'];
        save_obj = [num2str(data_nr),'_',file,'_objectives.mat'];
        save(save_pop, 'population', 'X_1', 'Y_1');
        save(save_obj, 'objectives', 'X_1', 'Y_2');

        load(save_pop)

    end

    function [sorted, index] = sort_objectives(objectives, i)

        [sorted,index] = sort(objectives(:,i));
    end

    function pop = decode_data(data, motion)

        if strcmpi(motion,'Lateral') || strcmpi(motion,'Eel')
            [pop_alpha, pop_omega, pop_delta] = decode_population(data,motion);
            pop = [pop_alpha,pop_omega,pop_delta];
        elseif strcmpi(motion,'Modified')
            [pop_alpha, pop_omega, pop_delta, g_func] = decode_population(data,motion);
            pop = [pop_alpha, pop_omega, pop_delta, g_func];
        elseif strcmpi(motion,'FourierSeries')
            [pop_omega, pop_delta, pop_coeff] = decode_population(data, motion);
            pop = [pop_omega, pop_delta, pop_coeff];
        elseif strcmpi(motion,'FourierSeries_OLD')
            [pop_delta, pop_coeff] = decode_population(data,motion);
            pop = [pop_delta, pop_coeff];
        else
            cd(return_path);
            error('Wrong motion pattern!')
        end
    end

    function [P_avg, v_avg] = get_objectives(objectives)
        P_avg = objectives(:,1);
        v_avg = -objectives(:,2);
    end

end



