function [P_obj, P_viol, sim_time, phi_avg, phi_d] = objective_function_evaluation(pop, motion)

    global  n g m_tot J D e theta0 theta0_dot p_CM0 p_CM0_dot friction_model ...
            ct cn mi_n lambda1 lambda2 lambda3 V_x V_y V V_1 V_2 V_current ...
            K Kp_joint Kd_joint vis_timestep visualize_motion violation_check          
    
    initModelParameters;
    initControllerParameters; 
        
    pop_size   = size(pop, 1);
    
    disp(pop_size)
    
    stop_time = 20;
    time_step = 0.1;
    sample_time = time_step:time_step:stop_time-time_step;
    swim_cycle = [0,sample_time,stop_time];
    
    % Matlab function used to measure how long the simulation takes
    t_start = tic;

    %% Initializations
    f = [];     % The objective set
    g = [];     % The constraint set
    
    % Saturation of torque input
    u_max=2.3*1000; %2.3Nm for Mamba joints

    % Saturation of angular speed
    phi_max=90*pi/180; %90 degrees

    % Saturation of angular speed
    phi_dot_max=429*pi/180; %429 degrees/sec
    
    % The upper bound for each parameters
    limits = [phi_max; phi_dot_max; u_max; alpha_max; omega_max; delta_max];

    % Constructs the initial values for the state vector.
    v0 = [theta0 ; p_CM0 ; theta0_dot ; p_CM0_dot];
    
%     for i = 1:pop_size
%        omega = pop(i,1);
%        disp(omega)
%     end
    
    %% Evaluate the individuals in the population
    % ------------ Solve the ODEs for the robot snake ----------------
    %options = odeset('RelTol', 1e-4, 'AbsTol', 1e-4);
    check_feasbile = @check_feasible_events;
    options = odeset('Events',check_feasbile,'RelTol',1e-3,'AbsTol',1e-3);
    parfor i = 1:pop_size
        [t, v] = ode23tb(@(t,v)calculate_v_dot(t, v, pop(i,:), ...
                n, e, friction_model, ct, cn, V_current, Kp_joint, ... 
                Kd_joint, K, u_max, J, D, V_x, V_y, V, V_1, V_2, m_tot, ... 
                mi_n, lambda1, lambda2, lambda3, motion), ...
                    swim_cycle, v0, options);
        
        sim_data(i).time = t;
        sim_data(i).state_vector = v;
    end
    % ----------------------------------------------------------------
    phi_d = [];
    phi_avg =  [];
    for i = 1:pop_size
        v                   = sim_data(i).state_vector;
        theta               = v(:, 1:n)';
        theta_dot           = v(:, n+3:2*n+2)';
        phi     = [];
        phi_dot = [];
        for k = 1:size(sim_data(i).time,1)
            phi     = [phi,D*theta(:,k)]; 
            phi_dot = [phi_dot,D*theta_dot(:,k)];
        end

        phi_avg(end+1) = sum(sum(abs(phi),2))/1000;
        phi_d(end+1) = sum(sum(abs(phi_dot),2))/1000;
%         disp([sum(sum(abs(theta),2)),theta_d(end)])
%         disp('=============================')
    end
    
    % Use the sim. data to compute the power and the input at each joint
    sim_data = simulation_data(n, Kp_joint, Kd_joint, u_max, D, sim_data,...
                                        pop_size, pop, motion);

    % Evaluate the objective function
    [forward_velocity,P_avg] = evaluate_mult_obj_func(pop_size,sim_data,n);
    
    %% ==================== Assign Objective Values =======================
    f = [P_avg', forward_velocity'];

%     alpha = pop(:,1);
%     omega = pop(:,2);
%     delta = pop(:,3);
%     I_avg = power_to_current(alpha,omega,delta);
%     f(:,1) = voltage_and_current(I_avg);

    P_obj   = double(f);

    %% ============== Assign Constraint violation values ==================
    P_viol  = double(zeros(pop_size,1)); 
    % Check for violations
    if violation_check
        a = 0;
        for i = 1:pop_size
            if sim_data(i).time(end) ~= stop_time
                P_viol(i) = double(floor((numel(swim_cycle)-numel(sim_data(i).time))/2));
                a = a + 1;
            end
        end
    end
    %% ============== Visualtization: Draw the snake robot ================
    if visualize_motion
        for i = 1:pop_size
            v = sim_data(i).state_vector;
            t = sim_data(i).time;
%             disp(t)
            [visualize_t,visualize_theta,visualize_p_CM] = init_visualization(v, t, n, vis_timestep);
            
            n_step = size(visualize_t,2);
            
            for k = 1:n_step
                initDrawSnakeRobot(visualize_t(k));
                drawSnakeRobot(visualize_theta(:,k), visualize_p_CM(:,k),'k');
                pause(vis_timestep*0.01);
                hold off
            end
        end
    end
    %% End of Simulation. Evaluate the compuation time
    % Displays how long the simulation took to complete
    sim_time = toc(t_start);
end

