function [regress] = regress_model(moea, motion)
    
    nr_of_poly = 16;

    obj = get_pareto_front(moea, motion);
    v_func = compute_poly_func(obj, nr_of_poly);
    [beta_val, v_bound] = get_pls_beta(moea, motion);

    regress = @(P_avg)(compute_regress_model(P_avg, v_func, beta_val, v_bound));
    
    function [objectives] = get_pareto_front(moea, motion)
       
        if strcmpi(moea,'HypE')
            if strcmpi(motion,'modified')
                pareto_front = load('Pareto_hype_modified');
                objectives = pareto_front.obj;
            end
%         elseif strcmpi(moea,'nsga2')

        end
    end

    function [v_func] = compute_poly_func(obj, n_p)
        p = polyfit(obj(:,1),obj(:,2),n_p);
        v_func = @(velocity)(polyval(p,velocity));
    end
    
    function [beta_val, v_bound] = get_pls_beta(moea, motion)
        if strcmpi(moea,'HypE')
            if strcmpi(motion,'modified')
                pls1 = load('hype_modified_beta1');
                pls2 = load('hype_modified_beta2');
                beta1 = pls1.hype_modified_beta1;
                beta2 = pls2.hype_modified_beta2;
                beta_val = {beta1, beta2};
                v_bound = [4.8, 9.4274];
            end
%         elseif strcmpi(moea,'nsga2')
            
        end
    end
    

    function [decision_var] = compute_regress_model(P_avg, v_func, beta_val, v_bound)
        
        
        v_avg = v_func(P_avg);
        
        decision_var = [];
        for i = 1:numel(v_avg)
            response = [];
            if v_avg(i) < 0 && v_avg(i) > max(v_bound)
                error('\n%s\n','Invalid value!!!')
            end

            for k = 1:numel(v_bound)
               if v_avg(i) <= v_bound(k)
                  response = [1,P_avg(i),v_avg(i)]*beta_val{k};
                  break;
               end
            end

            if isempty(response)
               [a,k] = min(abs(v_bound-v_avg(i)));
               response = [1,P_avg(i),v_avg(i)]*beta_val{k}; 
            end
            if response(1) >=0 && response(1) <= 90
                decision_var = [decision_var;response];
            end
        end
    end
end