function simp_drawSnakeRobot(t, px, py, theta, phi)
%
% Draws the snake robot in a graph window, also visualizing the current
%

global n l De D_bar e wx wy fignumber visualizeCMposition

% Sets figure properties.
window = figure(fignumber);

% Indicates that the erasing and redrawing of a rendering occurs off-screen
% to avoid flickering.
set(window, 'DoubleBuffer', 'on');
clf

% Sets the background color.
set(window, 'color', 'white');
hold on;
grid on;

axis('equal');
xlabel('X [m]');
ylabel('Y [m]');
% Rounds the time to two decimals precision.
t = round(t*100)/100;
title(['t = ' num2str(t)]);

% Calculates the position of the links.
ctheta = cos(theta);
stheta = sin(theta);
x = -l*ctheta*De + stheta*D_bar*phi + px*e;
y = -l*stheta*De - ctheta*D_bar*phi + py*e;

% Draws the snake robot.
for j = 1:n
    
    % Calculates the global frame coordinates of the
    % beginning of the next link
    startx = x(j, 1) - l/2*cos(theta);
    starty = y(j, 1) - l/2*sin(theta);
    
    % Calculates the global frame coordinates of the
    % end of the next link
    endx = x(j, 1) + l/2*cos(theta);
    endy = y(j, 1) + l/2*sin(theta);
    
    % The color of the link to be drawn
    link_color = 'b-';
    
    % Draws the next link.
    plot([startx ; endx], [starty ; endy], link_color, 'linewidth', 1.3);
end

% Draws the position of the CM (center of mass).
if visualizeCMposition
    plot(px, py, 'r*')
end

% Close-up rectangular view
axis([px-n*l-0.2, px+n*l+0.2, py-n*l-0.2, py+n*l+0.2]);

% draw the current
ax_x = px-n*l-0.2:0.5:px+n*l+0.2;
ax_y = py-n*l-0.2:0.5:py+n*l+0.2;
[ax_x, ax_y] = meshgrid(ax_x, ax_y);
quiver(ax_x,ax_y,wx*ones(size(ax_x)),wy*ones(size(ax_y)),0,'k')

hold off

end
