function sim_data = simulation_data(n, Kp_joint, Kd_joint, u_max, D, sim_data, pop_size, chromosome, motion)
    
    
%% Compute the power and controll input for each joints at each time sample
    for i = 1:pop_size
        u                   = [];
        P_joints_sim        = [];
        
        v                   = sim_data(i).state_vector;
        theta               = v(:, 1:n)';
        theta_dot           = v(:, n+3:2*n+2)';
        t_sim               = sim_data(i).time;
        
        phi     = [];
        phi_dot = [];
        for k = 1:size(t_sim,1) 
            phi     = [phi,D*theta(:,k)]; 
            phi_dot = [phi_dot,D*theta_dot(:,k)];
        end        

        switch motion
            case 'Lateral'
                % Calculates the actuator forces lateral undulaiton.
                alpha = chromosome(i,1);
                omega = chromosome(i,2);
                delta = chromosome(i,3);
                g_scale = [];
                coeff   = [];
                controller = @calculate_u_lateral;
            case 'Eel'
                % Calculates the actuator forces eel-like motion.
                alpha   = chromosome(i,1);
                omega   = chromosome(i,2);
                delta   = chromosome(i,3);
                g_scale = [];
                coeff   = [];
                controller = @calculate_u_eel;
            case 'Modified'
%                 alpha   = chromosome(i,1);
%                 omega   = chromosome(i,2);
%                 delta   = chromosome(i,3);
%                 g_scale = chromosome(i,4:end);
                alpha = 1;
                omega = 210*pi/180;
                delta   = chromosome(i,1);
                g_scale = chromosome(i,2:end);
                coeff   = [];
                controller = @calculate_u_modified;
            case 'FourierSeries'
                alpha   = [];
                omega   = chromosome(i,1);
                delta   = chromosome(i,2);
                g_scale = [];
                coeff   = chromosome(i,3:end);
                controller = @calculate_u_fourier;
            otherwise
                error('\n%s\n%s\n%s\n','@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',...
                                       '@@      Invalid motion pattern      @@',...
                                       '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@')
        end
 
        % Compute the controll input and the power
        for k = 1:size(t_sim,1)        
            t = t_sim(k);          

            u(:,end+1)  = controller(t, phi(:,k),phi_dot(:,k),...
                                        alpha,omega,delta,g_scale,coeff,...
                                        n,Kp_joint,Kd_joint,u_max);
                      
            
%             temp_P = u(:,end).*phi_dot(:,k);
%             P_joints=sum(temp_P);
            temp_P      = u(:,end).*phi_dot(:,k);
%             temp_P      = abs(u(:,end).*phi_dot(:,k));
            P_joints    = abs(sum(temp_P));
            P_joints_sim(end+1, 1)  = P_joints;
        end
        sim_data(i).P_joints = P_joints_sim;
        sim_data(i).input = u;
    end
end