%
% Starts the snake robot simulation.
%
clear all;
clc;
global visualize_motion previous_draw_time t_previous p_CM0

% The stop time for the simulation.
stop_time = 100;

% Initializes model parameters.
initModelParameters;

% Initializes controller parameters.
initControllerParameters;

disp('Starting simulation...')

% Matlab function used to measure how long the simulation takes
tic

% The simulation time of the previous time step
t_previous = 0;

% Constructs the initial values for the state vector.
v0 = [theta0 ; p_CM0 ; theta0_dot ; p_CM0_dot ; phi0 ; phi0_dot ; zeros(3, 1)];

% Starts the snake robot visualization
if 0*visualize_motion
    % The time of the previously drawn sample
    previous_draw_time = 0;

    % Draws the initial position of the snake robot
    drawSnakeRobot(0, theta0, p_CM0);
    
    % Waits for the figure window to display
    pause(0.1);
end
%return

if 1
  %Tspan=0:1:stop_time; 
options = odeset('Events', 'off', 'RelTol', 1e-4, 'AbsTol', 1e-4);
[t, v] = ode23tb('calculate_v_dot', [0 stop_time], v0, options);

disp('Simulation complete...')

% Displays how long the simulation took to complete
toc

disp(['Simulated time: ' num2str(t(end, 1)) 's'])

% Extracts the states from the state vector.
theta       = v(:, 1:n);
p_CM        = v(:, n+1:n+2);
theta_dot   = v(:, n+3:2*n+2);
p_CM_dot    = v(:, 2*n+3:2*n+4);
phi         = v(:, 2*n+5:3*n+3);
phi_dot     = v(:, 3*n+4:4*n+2);

end
%return

% Font size in figures.
label_font_size = 14;
axis_font_size = 12;
%figpos = [972   548   750   322];
%figpos = [980   401   752   365]; % ICRA 2011
figpos = [972   483   603   387]; % Avhandling

% % Plots the CM position.
% if 1
% 
% window = figure(7);
% set(window, 'color', 'white');
% plot(p_CM(:, 1), p_CM(:, 2), 'r--', 'linewidth', 2)
% %plot(p_CM(:, 1), p_CM(:, 2), 'b-')
% hold on
% xlabel('p_x [m]','FontSize',label_font_size);
% ylabel('p_y [m]','FontSize',label_font_size);
% set(gca,'FontSize',axis_font_size)
% 
% hold off
% 
% h = legend('Path of CM of robot', 'Reference path', 'location', 'best');
% set(h,'fontsize',12,'interpreter','latex');
% set(gcf,'Position',figpos)
% 
% legend('boxoff')
% end
% 
% % Plots theta in global frame.
% if 1
%     % Calculates average theta.
%     theta_av = zeros(size(t, 1), 1);
%     for i = 1:size(theta, 1)
%         theta_av(i, 1) = sum(theta(i, :))/n;
%     end
%     % Plots theta.
%     window = figure(16);
%     set(window, 'color', 'white');
%     plot(t, theta_av*180/pi, 'b')
%     grid on
%     xlabel('Time [s]','FontSize',label_font_size);
%     ylabel('$\theta$ [deg]','FontSize',label_font_size,'interpreter','latex')
%     %ylabel('$\overline{\theta}$ [deg]','FontSize',label_font_size,'interpreter','latex')
%     set(gca,'FontSize',axis_font_size)
%     set(gcf,'Position',figpos)
%     %set(gca,'XTick',[0 20 40 60 80]);
%     %ylim([-20 70])
% end
% 
% % Plots the position in y-axis.
% if 1
%     window = figure(17);
%     set(window, 'color', 'white');
%     plot(t, p_CM(:, 2)*100, 'b')
%     grid on
%     set(gca,'XGrid','off','YGrid','on')
%     xlabel('Time [s]','FontSize',label_font_size);
%     ylabel('p_y [cm]','FontSize',label_font_size);
%     set(gca,'FontSize',axis_font_size)
%     set(gcf,'Position',figpos)
% end
% 
% % Plots the CM velocity.
% if 1
%     window = figure(20);
%     set(window, 'color', 'white');
%     plot(t, p_CM_dot(:, 1)*100, 'b')
%     grid on
%     xlabel('Time [s]','FontSize',label_font_size);
%     ylabel('$\dot{p}_x$ [cm/s]','FontSize',label_font_size,'interpreter','latex');
%     set(gca,'FontSize',axis_font_size)
%     set(gcf,'Position',figpos)
%     set(gca,'XTick',[0 20 40 60 80]);
%     %ylim([-5 15])
%     
%     window = figure(21);
%     set(window, 'color', 'white');
%     plot(t, p_CM_dot(:, 2)*100, 'b')
%     grid on
%     xlabel('Time [s]','FontSize',label_font_size);
%     ylabel('$\dot{p}_y$ [cm/s]','FontSize',label_font_size,'interpreter','latex');
%     set(gca,'FontSize',axis_font_size)
%     set(gcf,'Position',figpos)
%     set(gca,'XTick',[0 20 40 60 80]);
%     %ylim([-5 15])
% end
% 
% if 1
%     
% % Plots amplitude of speed.
% speed = [];
% for i = 1:size(p_CM_dot, 1)
%     speed(i, 1) = sqrt(p_CM_dot(i, 1)^2+p_CM_dot(i, 1)^2);
% end
% window = figure(27);
% set(window, 'color', 'white');
% plot(t, speed, 'b')
% grid on
% xlabel('Time [s]','FontSize',14);
% ylabel('|v| [m/s]','FontSize',14);
% set(gca,'FontSize',12)
% %axis([0 stop_time 0 18])
% 
% end
% 
% 
% if 0
%     % Creates a combined plot from individual figures.
%     window = figs2subplots([16 1], [2 1]);
%     set(window, 'color', 'white');
% end
