function v_dot = calculate_v_dot(t, v, chromosome, n, e, friction_model,...
                                    ct, cn, V_current, Kp_joint, Kd_joint, K, u_max,...
                                    J, D, V_x, V_y, V, V_1, V_2, m_tot,...
                                    mi_n, lambda1, lambda2, lambda3, motion, with_omega)
%
% Calculates and returns v_dot.
%
% v = [theta ; p_CM ; theta_dot ; p_CM_dot]
%

% Extracts the states from the state vector
theta               = v(1:n, 1);
% p_CM                = v(n+1:n+2, 1);      % Used to calculate z
theta_dot           = v(n+3:2*n+2, 1);
p_CM_dot            = v(2*n+3:2*n+4, 1);

omega_max = 210*pi/180;

% Calculates the time step.
%delta_t = t - t_previous;

% Calculates diagonal matrices that are used in the proceeding calculations
sin_theta_diag = calculate_sin_theta_diag(theta,n);
cos_theta_diag = calculate_cos_theta_diag(theta,n);

% Calculates the joint angles
phi = D*theta;
% Calculates the joint angle velocities
phi_dot = D*theta_dot;

% Calculates the position of the links
% z = calculate_z(p_CM, e, K, sin_theta_diag, cos_theta_diag);

% Calculates the linear velocities of the links
z_dot = calculate_z_dot(theta_dot, p_CM_dot, sin_theta_diag, cos_theta_diag, e, K);
        
% Calculates translational friction forces.
fx = zeros(n, 1);
fy = zeros(n, 1);
if strcmp(friction_model, 'MORISON')
    [fx, fy] = calculate_f_drag(theta_dot, z_dot, n, sin_theta_diag, cos_theta_diag, ct, cn, V_current);
end

% Calculates rotational friction forces (friction torques).
% tau = zeros(n, 1);

switch motion
    case 'Lateral'
        % Calculates the actuator forces lateral undulaiton.
        if with_omega
            alpha = chromosome(1);
            omega = chromosome(2);
            delta = chromosome(3);
        else
            alpha = chromosome(1);
            omega = omega_max;
            delta = chromosome(2);
        end
        u = calculate_u_lateral(t, phi, phi_dot, alpha, omega, delta,[],[], n, Kp_joint, Kd_joint, u_max);
    case 'Eel'
        % Calculates the actuator forces eel-like motion.
        if with_omega
            alpha = chromosome(1);
            omega = chromosome(2);
            delta = chromosome(3);
        else
            alpha = chromosome(1);
            omega = omega_max;
            delta = chromosome(2);
        end
        u = calculate_u_eel(t, phi, phi_dot, alpha, omega, delta,[],[], n, Kp_joint, Kd_joint, u_max);
    case 'Modified'
%         alpha = chromosome(1);
%         omega = chromosome(2);
%         delta = chromosome(3);
%         g_scale = chromosome(4:end);
        if with_omega
            alpha = 1;
            omega = chromosome(1);
            delta = chromosome(2);
            g_scale = chromosome(3:end);
        else
            alpha = 1;
            omega   = omega_max;
            delta = chromosome(1);
            g_scale = chromosome(2:end);
        end
        u = calculate_u_modified(t,phi,phi_dot,alpha,omega,delta,g_scale,[],n,Kp_joint,Kd_joint,u_max);
    case 'FourierSeries'
        if with_omega
            omega = chromosome(1);
            delta = chromosome(2);
            coeff = chromosome(3:end);
        else
            omega = omega_max;
            delta = chromosome(1);
            coeff = chromosome(2:end);
        end
        u = calculate_u_fourier(t,phi,phi_dot,[],omega,delta,[],coeff,n,Kp_joint,Kd_joint,u_max);
    case 'MultiFourier'
        if with_omega
            omega = chromosome(1);
            delta = chromosome(2);
            coeff = chromosome(3:end);
        else
            omega = omega_max;
            delta = chromosome(1);
            coeff = chromosome(2:end);
        end
        u = calculate_u_multi_fourier(t,phi,phi_dot,[],omega,delta,[],coeff,n,Kp_joint,Kd_joint,u_max);
    otherwise
        error('\n%s\n%s\n%s\n','@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',...
                               '@@      Invalid motion pattern      @@',...
                               '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@')
end

% Calculates the acceleration of theta and p where p is the position of the CM
[phi_dot_dot, theta_dot_dot, p_CM_dot_dot] = calculate_q_dot_dot_CM_USR(t, theta_dot, p_CM_dot, fx, fy, u,...
                        n, J, D, e, V_x, V_y, V, V_1, V_2, sin_theta_diag,... 
                        cos_theta_diag, mi_n, lambda1, lambda2, lambda3, m_tot, K);

% Constructs the derivative of the state vector
% phi_o_dot_dot_dot=0;

v_dot = [theta_dot ; p_CM_dot ; theta_dot_dot ; p_CM_dot_dot];

% %Calculate the power input to the system
% temp_P=u.*phi_dot;
% P_joints=sum(temp_P);
% 

end
