function [violated] = check_feasible_constraints(n_links, D, sim_data, limits, chromosome)
    
%     phi_max     = limits(1);
%     phi_dot_max = limits(2);
%     u_max       = limits(3);
%     alpha_max   = limits(4);
%     omega_max   = limits(5);
%     delta_max   = limits(6);
    
    % Saturation of torque input
    u_max=2.3*1000; %2.3Nm for Mamba joints

    % Saturation of angular speed
    phi_max=90*pi/180; %90 degrees

    % Saturation of angular speed
    phi_dot_max=429*pi/180; %429 degrees/sec

%     v            = sim_data.state_vector;
%     phi          = v(:, 2*n_links+5:3*n_links+3);
%     phi_dot      = v(:, 3*n_links+4:4*n_links+2);
    v                   = sim_data.state_vector;
    theta               = v(:, 1:n_links)';
    theta_dot           = v(:, n_links+3:2*n_links+2)';
    t_sim               = sim_data.time;

    phi     = [];
    phi_dot = [];
    for k = 1:size(t_sim,1) 
        phi     = [phi,D*theta(:,k)]; 
        phi_dot = [phi_dot,D*theta_dot(:,k)];
    end        
    
    u            = sim_data.input;
    a 			 = 0;
    
    % The total number of constraints is calculated by taking the
    % consideration of each time step in the simulation, where the
    % number of time steps is the size(u,2)
    n_time_steps = size(u,2);
    
    % Create a constraint set g. Note that the constraints violation are
    % considered at each time sample from [0,stop_time] calcuted in ode23tb
%     total_constraints = 6*(n_links-1)*n_time_steps + 6;   
%     g  = a*ones(total_constraints,1);
    violated = 0;
    
    %% Description/Outline
        
    % Compute the constraints of the snake at each time sample
    for k = 1:n_time_steps
        for i = 1:n_links-1
            % The lower bound of phi
            if -(phi(i,k) + phi_max) > 0
                violated = violated + 1;
%                 g((i-1)*(n_links-1)*6+(k-1)*6+1) = a + (-(phi(i,k)+phi_max));
            end
            % The upper bound of phi
            if phi(i,k)-phi_max > 0
                violated = violated + 1;
%                 g((i-1)*(n_links-1)*6+(k-1)*6+2) = a + phi(i,k)-phi_max;
            end
            % The lower bound of phi_dot
            if -(phi_dot(i,k) + phi_dot_max) > 0
                violated = violated + 1;
%                 g((i-1)*(n_links-1)*6+(k-1)*6+3) = a + (-(phi_dot(i,k)+phi_dot_max));
            end
            % The upper bound of phi_dot
            if phi_dot(i,k)-phi_dot_max > 0
                violated = violated + 1;
%                 g((i-1)*(n_links-1)*6+(k-1)*6+4) = a + phi_dot(i,k)-phi_dot_max;
            end
            % The lower bound of the joint input
            if -(u(i,k) + u_max) > 0
                violated = violated + 1;
%                 g((i-1)*(n_links-1)*6+(k-1)*6+5) = a + (-(u(i,k)+u_max));
            end
            % The upper bound of the joint input
            if u(i,k)-u_max > 0
                violated = violated + 1;
%                 g((i-1)*(n_links-1)*6+(k-1)*6+6) = a + u(i,k)-u_max;
            end
        end
    end            
end