function [P_obj, P_viol, sim_time] = objective_function_evaluation(gait, motion, with_omega)

    global  n g m_tot J D e theta0 theta0_dot p_CM0 p_CM0_dot friction_model ...
            ct cn mi_n lambda1 lambda2 lambda3 V_x V_y V V_1 V_2 V_current ...
            K Kp_joint Kd_joint           
    
    initModelParameters;
    initControllerParameters; 
        
    chromosome = [];
    for i=1:numel(gait)
        chromosome = [chromosome,cell2mat(gait{i})'];
    end
    
%     chromosome =[cell2mat(gait{1})',cell2mat(gait{2})',cell2mat(gait{3})'];
    pop_size   = size(chromosome, 1);
    
%     disp(chromosome)
    
    % The stop time for the simulation.
    stop_time = 20;
    time_step = 0.1;
    sample_time = time_step:time_step:stop_time-time_step;
    swim_cycle = [0,sample_time,stop_time];
    % Matlab function used to measure how long the simulation takes
    t_start = tic;

    %% Initializations
    f = [];     % The objective set
    P_viol = [];     % The constraint set
    
    % Saturation of torque input
    u_max=2.3*1000; %2.3Nm for Mamba joints

    % Saturation of angular speed
    phi_max=90*pi/180; %90 degrees

    % Saturation of angular speed
    phi_dot_max=429*pi/180; %429 degrees/sec
    
    % The upper bound for each parameters
    limits = [phi_max; phi_dot_max; u_max; alpha_max; omega_max; delta_max];

    % Constructs the initial values for the state vector.
    v0 = [theta0 ; p_CM0 ; theta0_dot ; p_CM0_dot];
    
%     for i = 1:pop_size
%        omega = chromosome(i,1);
%        disp(omega)
%     end
    
    %% Evaluate the individuals in the population
    % ------------ Solve the ODEs for the robot snake ----------------
    %options = odeset('RelTol', 1e-4, 'AbsTol', 1e-4);
%     check_feasbile = @check_feasible_events;
%     options = odeset('Events',check_feasbile,'RelTol',1e-3,'AbsTol',1e-3);
    options = odeset('RelTol',1e-3,'AbsTol',1e-3);
    parfor i = 1:pop_size
        [t, v] = ode23tb(@(t,v)calculate_v_dot(t, v, chromosome(i,:), ...
                n, e, friction_model, ct, cn, V_current, Kp_joint, ... 
                Kd_joint, K, u_max, J, D, V_x, V_y, V, V_1, V_2, m_tot, ... 
                mi_n, lambda1, lambda2, lambda3, motion, with_omega), ...
                    swim_cycle, v0, options);
        
        sim_data(i).time = t;
        sim_data(i).state_vector = v;
    end
    % ----------------------------------------------------------------
    
    % Use the sim. data to compute the power and the input at each joint
    sim_data = simulation_data(n, Kp_joint, Kd_joint, u_max, D, sim_data,...
                                        pop_size, chromosome, motion, with_omega);

    % Evaluate the objective function
    [forward_velocity,P_avg] = evaluate_mult_obj_func(pop_size,sim_data,n);
    % [fitness, violated] = static_penalty(n,sim_data,limits,chromosome);
    % [penalty, violation] = dynamic_penalty(n,sim_data,limits,chromosome,current_gen);  
        
    % Concatenate the objective and the constraint set into the population
    f = [P_avg', -forward_velocity'];
    P_obj   = double(f);

    % -------------------- Compute the constraints  -----------------------
    % ========================== Method 1 =================================
    for i = 1:pop_size
        [violated] = check_feasible_constraints(n,D,sim_data(i),limits,chromosome(i,:));
        P_viol = [P_viol;violated];
    end

    % ========================== Method 2 =================================
%     P_viol  = double(zeros(pop_size,1));    
%     a = 0;
%     for i = 1:pop_size
%         if sim_data(i).time(end) ~= stop_time
% %             P_obj(i,:) = double([10000,0]);
%             P_viol(i) = double(floor((numel(swim_cycle)-numel(sim_data(i).time))/2));
%             a = a + 1;
%         end
%     end    
    % ---------------------------------------------------------------------
    
%     disp(P_obj)
    
%     disp(a)
    
    % Displays how long the simulation took to complete
    sim_time = toc(t_start);
end

