import load_utility as load
import write_utility as write
import convert_gz_to_json as convert

import numpy as np
import os

import zipfile
import zlib

def init_dict(data, keys):

    for i in range(len(keys)):
        it = data
        for key in keys[i]:
            if key in it:
                it = it[key]
            else:
                it[key] = {}
                it = it[key]
    return data

def merge_information(data, keys, outputfile):
    infoText = ''
    for key in keys:
        it = data[key[0]]
        for i in range(1,len(key)):
            it = it[key[i]]
        n_data = len(it["objectives"])
        infoText += "%d: " % n_data + key[0] + ", " + key[1] + ", " + key[2] + \
                        ", " + key[3] + ", " + key[4] + ", " + key[5] + '\n'

    with open(outputfile, 'w') as fp:
        fp.write(infoText)
    fp.close()

def merge_result_data(result1, info1, result2, info2, mergedfile, mergedinfo):

    print("## Result json-file 1:")
    data1 = load.json_file(result1)
    print("\n## Result json-file 2:")
    data2 = load.json_file(result2)

    print("\n### Merging the files...")
    keys1, n_data1 = load.info_file(info1)
    keys2, n_data2 = load.info_file(info2)

    merged = init_dict({}, keys1)
    merged = init_dict(merged, keys2)

    for i in range(len(keys1)):
        key_exist = False
        for j in range(len(keys2)):
            if keys1[i] == keys2[j]:
                it      = merged[keys1[i][0]]
                result1 = data1[keys1[i][0]]
                result2 = data2[keys1[i][0]]
                for k in range(1,len(keys1[i])):
                    it      = it[keys1[i][k]]
                    result1 = result1[keys1[i][k]]
                    result2 = result2[keys1[i][k]]

                it["population"] = result1["population"] + result2["population"]
                it["objectives"] = result1["objectives"] + result2["objectives"]

                key_exist = True
                del keys2[j]
                break

        if not key_exist:
            it      = merged[keys1[i][0]]
            result1 = data1[keys1[i][0]]
            for k in range(1,len(keys1[i])):
                it      = it[keys1[i][k]]
                result1 = result1[keys1[i][k]]

            it["population"] = result1["population"]
            it["objectives"] = result1["objectives"]

    if keys2:
        for key in keys2:
            it = merged[key[0]]
            result = data2[key[0]]
            for i in range(1,len(key)):
                it = it[key[i]]
                result = result[key[i]]

            it["population"] = result["population"]
            it["objectives"] = result["objectives"]

    write.json_file(merged, mergedfile)
    # write.gz_file(merged, mergedfile)
    merge_information(merged, keys1+keys2, mergedinfo)
    print("====================== Merging Done! =======================")

# def pre_process_data(input1, output1, input2, output2):
#     print("#### Convert the .gz files to .json files:")
#     print("## gz-file 1:")
#     convert.gz_to_json(input1, output1)
#     print("\n## gz-file 2:")
#     convert.gz_to_json(input2, output2)
#     print("#### Converting Done!")
#     print("############################################################")

def compressToZip(result_file, result_info, zip_file):

    # Try the deflated compression mode
    try:
        compression = zipfile.ZIP_DEFLATED
    except:
        compression = zipfile.ZIP_STORED

    # Compression modes
    modes = { zipfile.ZIP_DEFLATED: 'deflated', zipfile.ZIP_STORED: 'stored'}

    print('##  Creating compressed', zip_file ,'file...')
    zf = zipfile.ZipFile(zip_file, mode='w')
    try:
        print('##  The compression mode is:',  modes[compression])
        print('##  Adding files:'),
        print('##  \t- ',result_file)
        print('##  \t- ',result_info)
        zf.write(result_file, compress_type=compression)
        zf.write(result_info, compress_type=compression)
    finally:
        print('##  Closing the .zip file!')
        zf.close()

if __name__ == "__main__":
    ###########################################################################
    os.chdir("../extended_model")
    ###########################################################################
    # Merge .json result files and .txt info files
    print("====================== Begin Merging! ======================\n")
    merge_result_data("data2.json", "info2.txt", \
                        "data3.json", "info3.txt", \
                            "pop.json", "popInfo.txt")

    # Compres the merged files to a compressed .zip file
    # compressToZip("mergedData.json", "mergedInfo.txt", "mergedData.zip")
