function [varargout] = decode_population(population, motion)
    
    global n_var n

    alpha_min   = 0;
    alpha_max   = 90;
    omega_min   = 0;
    omega_max   = 210;
    delta_min   = 0;
    delta_max   = 90;

    % gen lengths
    alpha_gen   = 7;
    omega_gen   = 7;
    delta_gen   = 7;

    % Fourier series
    coeff_max   = 90;
%     a_null_max  = 90;
    
    %########################## Length of each genes ##########################

    % Modified motion
    g_gen       = 7;
    g2_gen      = 7;

    % Fourier series
%     n_coefficients      = 2*4;
    fourier_gen  = 7;     % gen length for each coefficients
%     a_null       = 7;

    output = {};
    if strcmpi(motion,'Lateral') || strcmpi(motion,'Eel')
        if mod(n_var,2) ~= 0
            [alpha,omega,delta] = decode_gait(population);
            output{1} = alpha;
            output{2} = omega;
            output{3} = delta;
        else 
            output{1} = double((alpha_max-alpha_min).*population(:,1))./(bitshift(1,alpha_gen)-1) + alpha_min;
            output{2} = double((delta_max-delta_min).*population(:,2))./(bitshift(1,delta_gen)-1) + delta_min;
        end   
    elseif strcmpi(motion,'Modified1')
        [alpha,omega,delta] = decode_gait(population(:,1:3));
        g_scale = decode_g(population(:,4:end));
        output{1} = alpha;
        output{2} = omega;
        output{3} = delta;
        output{4} = g_scale;
    elseif strcmpi(motion,'Modified')
        if mod(n_var-(n-1),2) ~= 0
            output{1} = double((delta_max-delta_min).*population(:,1))./(bitshift(1,delta_gen)-1) + delta_min;
            output{2} = decode_g2(population(:,2:end));
        else
            output{1} = double((omega_max-omega_min).*population(:,1))./(bitshift(1,omega_gen)-1) + omega_min;
            output{2} = double((delta_max-delta_min).*population(:,2))./(bitshift(1,delta_gen)-1) + delta_min;
            output{3} = decode_g2(population(:,3:end));
        end
    elseif strcmpi(motion,'FourierSeries')
        if mod(n_var,2) ~= 0
            output{1} = double((delta_max-delta_min).*population(:,1))./(bitshift(1,delta_gen)-1) + delta_min;
            output{2} = decode_fourier(population(:,2:end));
%             output{1} = double((omega_max-omega_min).*population(:,1))./(bitshift(1,omega_gen)-1) + omega_min;
%             output{2} = double((delta_max-delta_min).*population(:,2))./(bitshift(1,delta_gen)-1) + delta_min;
%             output{2} =[output{2}, decode_fourier(population(:,3))]; 
        else
            output{1} = double((omega_max-omega_min).*population(:,1))./(bitshift(1,omega_gen)-1) + omega_min;
            output{2} = double((delta_max-delta_min).*population(:,2))./(bitshift(1,delta_gen)-1) + delta_min;
            output{3} = decode_fourier(population(:,3:end));            
        end
    elseif strcmpi(motion,'FourierSeries_OLD')
%         omega = double((omega_max-omega_min).*population(:,1))./(bitshift(1,omega_gen)-1) + omega_min;
        delta = double((delta_max-delta_min).*population(:,1))./(bitshift(1,delta_gen)-1) + delta_min;
        coeff = decode_fourier(population(:,2:end));
        output{1} = delta;
        output{2} = coeff;
    elseif strcmpi(motion,'MultiFourier')
        
        if mod(n_var,2) ~= 0
            output{1} = double((delta_max-delta_min).*population(:,1))./(bitshift(1,delta_gen)-1) + delta_min;
            output{2} = decode_fourier(population(:,2:end));
        else
            output{1} = double((omega_max-omega_min).*population(:,1))./(bitshift(1,omega_gen)-1) + omega_min;
            output{2} = double((delta_max-delta_min).*population(:,2))./(bitshift(1,delta_gen)-1) + delta_min;
            output{3} = decode_fourier(population(:,3:end));
        end
        
    elseif strcmpi(motion,'MultiFourier1')
        
    end
    
    if max(nargout,1) > numel(output)
       error('Error occured in decode_population.\nToo many output arguments\n'); 
    else
        for k = 1:numel(output)
            varargout{k} = output{k};
        end
    end
    
    function [alpha,omega,delta] = decode_gait(population)
        
        alpha = double((alpha_max-alpha_min).*population(:,1))./(bitshift(1,alpha_gen)-1) + alpha_min;
        omega = double((omega_max-omega_min).*population(:,2))./(bitshift(1,omega_gen)-1) + omega_min;
        delta = double((delta_max-delta_min).*population(:,3))./(bitshift(1,delta_gen)-1) + delta_min;
    end

    function [g_scale] = decode_g(population)
        g_scale = zeros(size(population));
        for i = 1:size(population,2)
            g_scale(:,i) = double(population(:,i))./(bitshift(1,g_gen)-1);
        end
    end

    function [g_scale] = decode_g2(population)
        g_scale = zeros(size(population));
        for i = 1:size(population,2)
            g_scale(:,i) = double((alpha_max-alpha_min)*population(:,i))./(bitshift(1,g2_gen)-1) + alpha_min;
        end
    end

    function [fourier_coeff] = decode_fourier(population)
        fourier_coeff = zeros(size(population));
        gen_max = (bitshift(1,fourier_gen)-1);
        mean_bound = int32(gen_max/2);
        for i = 1:size(population,1)
            ind = population(i,:);
            for j = 1:numel(ind)
                if ind(j) >= mean_bound && ind(j) <= mean_bound+1
                    fourier_coeff(i,j) = 0;
                else
                    fourier_coeff(i,j) = coeff_max*(2*double(ind(j))./(gen_max-1) - 1);
                end
            end
        end 

    end

end