% function load_result()

clear all

global omega_max return_path n_var n
omega_max = 210;
n         = 10;

addpath(pwd)
return_path = pwd;
cd('../extended_model/mat_files');

%% Specify the data file to read
moea        = 'HypE';
motion      = 'multiFourier';
pop_size    = 'pop500';
gen_size    = 'gen200';
crossover   = 'cross070';
mutation    = 'mut0150';
type = '';
% type = '';

gen_nr = '200';

file = [ moea, '_', motion, '_', pop_size, '_', gen_size, '_',...
              crossover,'_', mutation];

%% Read the data file
[population, objectives, data_nr, pop_all, obj_all] = load_file(file, gen_nr);
n_var = size(population,2);

%% Sort Objectives
[sorted_obj, indexes] = sort_objectives(objectives, 1);
population = population(indexes,:);
objectives = objectives(indexes,:);

% population(end-20,:)

%% Decode the population
decoded_pop = decode_data(population, [motion,type]);

% objectives

% population
% 
% decoded_pop

% decoded_pop
% decoded_pop = decode_data(population, 'FourierSeries_OLD');

% 
%% Frequency distribution
% omega_index = 1;
% freq_start = 10;
% freq_end = 220;
% % hFig = figure(45);
% hFig = figure(1);
% set(hFig, 'Position', [300 200 600 500])
% hold on
% freq_dist(decoded_pop, '$\omega$ [Nm]', omega_index, 10, 220, [0,0.9], motion)
% 
% % freq_dist(decoded_pop, '$\delta$ [Nm]', omega_index, 10, 220, [0,0.25], motion)
% xlim([freq_start,freq_end]);
% % set(gca,'xtick',[-5, 18, 36, 54, 72, 90]);
% % legend('Location','ne');
% set(gca,'xtick',[0, 50, 100, 150, 210]);

%% Get the objective values
[P_avg, v_avg] = get_objectives(objectives);

%% Plot objectives
hFig = figure(47);
set(hFig, 'Position', [300 200 800 600])
% set(hFig, 'Position', [300 200 600 500])
% subplot(3,3,2)
hold on
if strcmpi(moea, 'nsga2') 
    plot_motion = 'NSGA-II';
else
    plot_motion = 'HypE';
end

plot_objectives(objectives, gen_nr, plot_motion, motion)
% xlim([0,30]);
% ylim([0,0.85]);
xlim([0,45]);
ylim([0,1]);
% 
% xlim([5,15]);
% ylim([0.6,0.82]);


%% Plot Time lapse
% time_lapse(obj_all, moea, motion)

%% Plot the motion pattern reference
% i = find(decoded_pop(:,omega_index) == omega_max);
% decoded_pop1 = [decoded_pop(1,:);decoded_pop(end-20,:)];
% decoded_pop1 = [decoded_pop(end-10,:)];
decoded_pop1 = [decoded_pop(308,:)];
% decoded_pop1 = decoded_pop;
plot_phi(decoded_pop1, 0, 0.1, 20, 0.3, [0,10],[-90,90],[motion,type]);
% i = (200:300)';
% plot_phi(decoded_pop(i,:), 0, 0.1, 20, 0.3, [0,20],[-80,80],'FourierSeries_OLD');

%% Analyzed with PCA in Unscrambler
% [population, decoded_pop, objectives] = analyzed_data(population, decoded_pop, objectives);


% Save population and objectives to .mat file (Used for further analysis)
obj = [objectives(:,1),-objectives(:,2)*10];   % m -> dm
save_file(file, data_nr, decoded_pop, obj, [motion,type])
% % save_file(file, data_nr, decoded_pop, obj, 'FourierSeries_OLD');

%
% 
cd(return_path);
cd('../pareto_fronts');
obj = [objectives(:,1),-objectives(:,2)*10];
file_name = ['pareto','_',[motion]];
save(file_name, 'obj');

cd(return_path);

%% ########################################################################
%% =========================== Time Lapse =================================
function time_lapse(obj_all, moea, motion)
    for gen = 2:numel(obj_all)
        hold off
        obj = obj_all{gen};
        plot_objectives(obj, num2str(gen-1), moea, motion)
        pause(0.05);
    end
end
%% =========================== Plot Objectives ============================
function plot_objectives(obj, gen_nr, moea, motion)
    
    b = [ 0,0.4470,0.7410];
    r = [0.8500,0.3250,0.0980];

%     figure(45)
    % Plot objectives
%     scatter(obj(:,1),-obj(:,2), 60, '.', 'DisplayName', ['MOEA: ', moea, ', Motion: ', motion]);
    scatter(obj(:,1),-obj(:,2), 'DisplayName', ['MOEA: ', moea, ', Motion: ', motion]);
%     scatter(obj(:,1),-obj(:,2), 50, '.','MarkerEdgeColor',r, 'DisplayName', ['MOEA: ', moea, ', Motion: ', motion, ', GenNr: ', gen_nr]);
%     scatter(obj(:,1),-obj(:,2), 60,'o','MarkerEdgeColor',r,'DisplayName', ['MOEA: ', moea, ', Motion: ', motion, ', GenNr: ', gen_nr]);

%     Title
%     title_text = sprintf("Generation: %s", gen_nr);
%     title(title_text)
    
    % Labels
    xlabel('$P_{avg}$ [W]','FontSize',14,'interpreter','latex');
    ylabel('$\bar{\upsilon}$ [m/s]','FontSize',14,'interpreter','latex'); 
    
    % Legend
    le = legend('Location', 'se');
    set(le,'FontSize',12,'interpreter','latex')
    
    % Figure position
%     figpos = [972   483   603   387];
%     set(gcf,'Position',figpos)
    grid on
end
%% ======================= Frequency Distribution =========================
function freq_dist(population, var_name, var_nr, freq_start, freq_end, y_axis, motion)
    
    frequency = (freq_start:freq_end)';
    a = population(:, var_nr);
    distribution = zeros(numel(frequency),1);
%     distribution = zeros(numel(a),1);

    for i = 1:numel(frequency)
        distribution(i) = numel(find(round(a) == frequency(i)));
    end
    distribution = distribution./sum(distribution);
%     disp(distribution)
    plot(frequency, distribution, 'DisplayName', ['Motion: ', motion]);
    ylabel('Population In [\%]', 'Interpreter', 'latex', 'Fontsize', 14)
    xlabel(var_name, 'Interpreter', 'latex', 'Fontsize', 14)
%     xlim([freq_start,freq_end])
    ylim(y_axis)
    
    grid on
    
        % Legend
    le = legend('Location', 'nw');
    set(le,'FontSize',12,'interpreter','latex')
    xlim([freq_start,freq_end]);
    xticks = ([freq_start,210,freq_end]);
    set(gca,'xtick',[0, 50, 100, 150, 210]);
    set(gca,'xtick',[0, 18, 36, 54, 72, 90]);
    set(gca,'Xtick',freq_start:10:freq_end);

%     Ticks: Label with line break!
%     figure(3)
%     y = [1,2,3];
%     bar(y)
%     xticks = ([1,2,3]);
%     a = 'Yo\newlineMorten';
%     xticklabels({a, 'wooop', 'woop'}); 
end

%% List over the analyzed data in Unscrambler
function [population, decoded_pop, objectives] =  analyzed_data(population, decoded_pop, objectives)
    i = analyzed_list();
    population = population(i,:);
    decoded_pop = decoded_pop(i,:);
    objectives = objectives(i,:);
end

%% ========================= Utility Functions ============================
function [population, objectives, data_nr, pop_all, obj_all] = load_file(file_read, gen_nr)
    
    global return_path

    if exist([file_read,'.mat'],'file') == 2
        data = load(file_read);      
    else
        cd(return_path);
        error('############### Invalid file ####################')
    end
    
    i1 = [];
    i2 = [];
    temp1 = [];
    temp2 = [];
    for i = 1:numel(data.population)
        a = size(data.population{i}{end},2);
        if any(temp1 == a) || isempty(temp1)
            temp1(end+1) = a;
            i1(end+1) = i;
        else
            temp2(end+1) = a;
            i2(end+1) = i;
        end
    end

    fprintf('\n');
    disp('==================================');
    fprintf('##\t Select data:\n');
    if ~isempty(i2)
        if temp1(1) < temp2(1)
           temp = i1;
           i1 = i2;
           i2 = temp;
        end
        fprintf('##\t\t- === With omega ===\n');
        for i = i1
            output_text = sprintf('##\t\t\t= Sample %d\n',i);
            fprintf(output_text);
        end
        fprintf('##\t\t- === Without omega ===\n');
        for i = i2
            output_text = sprintf('##\t\t\t= Sample %d\n',i);
            fprintf(output_text);
        end
    else
        fprintf('##\t\t- === Undefined: With or without omega ===\n');
        for i = i1
            output_text = sprintf('##\t\t\t= Sample %d\n',i);
            fprintf(output_text);
        end
        
    end
%     disp(numel(data.population))
    
    data_nr = 0;
    while isempty(data_nr) || ((data_nr > numel(data.population)) || (data_nr < 1)) 
        text_input = sprintf('Select a nr. between 1-%d: ', numel(data.population));
        data_nr = int16(input(text_input)); 
    end       
    
    pop_all = data.population{data_nr};
    obj_all = data.objectives{data_nr};
    
    if strcmpi(gen_nr,'end')
        population = double(data.population{data_nr}{end});
        objectives = double(data.objectives{data_nr}{end});
    else
        if str2double(gen_nr) <= numel(data.population{data_nr})
            population = double(data.population{data_nr}{str2double(gen_nr)});
            objectives = double(data.objectives{data_nr}{str2double(gen_nr)});
        else
            cd(return_path);
            error('Invalid generation')
        end
    end
end

function save_file(file, data_nr, pop, obj, motion)

    global return_path n_var n omega_max

    population  = pop;
    X_1         = char([]);
    Y_1         = char([]);
    
    objectives  = obj;
    X_2         = char([]);
    Y_2         = char([]);
    
    velocity    = double([0,0.2,0.4,0.6,0.8,1])*10;
    range       = [];
    
%     omega_index = 2;
    % ============ Remove Outliers =============
%     omega_opt = omega_max;
%     i = find(population(:,omega_index) == omega_opt);
%     population = population(i,:);
%     objectives = objectives(i,:);
    
    % ============= omega not equal omega_max ===============
%     i = find(population(:,omega_index) ~= omega_max);
%     if ~isempty(i)
%         disp('Individuals with no omega max')
%         disp(i)
%     end
    
    str_len = 4;
    for i = 1:size(population,1)
        a = sprintf('%d',i);
        for j =1:str_len-numel(a)
            a = [a,' '];
        end
        X_1(i,:) = a;     
    end
    
    if strcmpi(motion,'Lateral') || strcmpi(motion,'Eel')
        if mod(n_var,2) ~= 0
            Y_1 = ['alpha'; 'omega'; 'delta'];
        else
            Y_1 = ['alpha'; 'delta'];
        end
        Y_2 = ['Avgerage Power  '; 'Average Velocity'];
    elseif strcmpi(motion,'Modified1')
     	Y_1 = ['alpha'; 'omega'; 'delta'; 'g1   ';'g2   ';'g3   ';'g4   ';'g5   ';'g6   ';'g7   ';'g8   ';'g9   '];
%         Y_1 = ['delta'; 'g1   ';'g2   ';'g3   ';'g4   ';'g5   ';'g6   ';'g7   ';'g8   ';'g9   '];
        Y_2 = ['Avgerage Power  '; 'Average Velocity'];
    elseif strcmpi(motion,'Modified')
        if mod(n_var-(n-1),2) ~= 0
            Y_1 = ['delta'; 'g1   ';'g2   ';'g3   ';'g4   ';'g5   ';'g6   ';'g7   ';'g8   ';'g9   '];
        else
            Y_1 = ['omega'; 'delta'; 'g1   ';'g2   ';'g3   ';'g4   ';'g5   ';'g6   ';'g7   ';'g8   ';'g9   '];
        end
        Y_2 = ['Avgerage Power  '; 'Average Velocity'];        
    elseif strcmpi(motion,'FourierSeries')
        if mod(n_var,2) ~= 0
            Y_1 = ['delta';[]];
            n_coeff = numel(population(1,2:end))/2;
        else
            Y_1 = ['omega';'delta'];
            n_coeff = numel(population(1,3:end))/2;
        end
        str_len = numel(Y_1(1,:));

        a_coeff = [];
        b_coeff = [];
        for i = 1:n_coeff
            str1 = sprintf('a%d',i);
            str2 = sprintf('b%d',i);
            % Add spacing
            for j = 1:str_len-numel(str1)
                str1 = [str1, ' '];
                str2 = [str2, ' ' ];
            end
            a_coeff = [a_coeff; str1];
            b_coeff = [b_coeff; str2];
        end
        Y_1 = [Y_1;a_coeff;b_coeff];
        Y_2 = ['Avgerage Power  '; 'Average Velocity'];
%     elseif strcmpi(motion,'FourierSeries_OLD')
%         Y_1 = ['delta';[]];
%         str_len = numel(Y_1(1,:));
%         n_coeff = numel(population(1,2:end))/2;
%         a_coeff = [];
%         b_coeff = [];
%         for i = 1:n_coeff
%             str1 = sprintf('a%d',i);
%             str2 = sprintf('b%d',i);
%             % Add spacing
%             for j = 1:str_len-numel(str1)
%                 str1 = [str1, ' '];
%                 str2 = [str2, ' ' ];
%             end
%             a_coeff = [a_coeff; str1];
%             b_coeff = [b_coeff; str2];
%         end
%         Y_1 = [Y_1;a_coeff;b_coeff];
%         Y_2 = ['Avgerage Power  '; 'Average Velocity'];
    elseif strcmpi(motion,'MultiFourier')       
        if mod(n_var,2) ~= 0
            Y_1 = ['delta';[]];
            n_coeff = numel(population(1,2:end))/2;
        else
            Y_1 = ['omega';'delta'];
            n_coeff = numel(population(1,3:end))/2;
        end
        
        str_len = numel(Y_1(1,:));        
        
        a_coeff = [];
        b_coeff = [];
        for i = 1:n_coeff
            str1 = sprintf('a%d',i);
            str2 = sprintf('b%d',i);
            % Add spacing
            for j = 1:str_len-numel(str1)
                str1 = [str1, ' '];
                str2 = [str2, ' ' ];
            end
            a_coeff = [a_coeff; str1];
            b_coeff = [b_coeff; str2];
        end
        Y_1 = [Y_1;a_coeff;b_coeff];
        Y_2 = ['Avgerage Power  '; 'Average Velocity'];
    else
        error('No variable naming done yet!!!!!')
        cd(return_path);
    end  

%     disp(population(:,4:5));
%     range = {};
    k = 1;
    for i = 1:numel(velocity)-1
        indexes = find(objectives(:,2) >= velocity(i) & objectives(:,2) <= velocity(i+1));
        if ~isempty(indexes)
           range{k} = indexes; 
           k = k +1;
        end
    end
    
    
    for i = 1:numel(range)
       disp(['Range',num2str(i),': ']);
       disp(range{i}(end))
    end
   
    save_pop = [num2str(data_nr),'_',file,'_population.mat'];
    save_obj = [num2str(data_nr),'_',file,'_objectives.mat'];
    save(save_pop, 'population', 'X_1', 'Y_1');
    save(save_obj, 'objectives', 'X_1', 'Y_2');

    load(save_pop)

end

function [sorted, index] = sort_objectives(objectives, i)

    [sorted,index] = sort(objectives(:,i));
end

function pop = decode_data(data, motion)
    
    global return_path n_var n

    if strcmpi(motion,'Lateral') || strcmpi(motion,'Eel')
        if mod(n_var,2) ~= 0
            [pop_alpha, pop_omega, pop_delta] = decode_population(data,motion);
            pop = [pop_alpha,pop_omega,pop_delta];
        else 
            [pop_alpha, pop_delta] = decode_population(data,motion);
            pop = [pop_alpha,pop_delta];
        end
    elseif strcmpi(motion,'Modified1')
        [pop_alpha, pop_omega, pop_delta, g_func] = decode_population(data,motion);
        pop = [pop_alpha, pop_omega, pop_delta, g_func];
    elseif strcmpi(motion,'Modified')
        if mod(n_var-(n-1),2) ~= 0
            [pop_delta, g_func] = decode_population(data,motion);
            pop = [pop_delta, g_func];
        else
            [pop_omega, pop_delta, g_func] = decode_population(data,motion);
            pop = [pop_omega, pop_delta, g_func];
        end
    elseif strcmpi(motion,'FourierSeries')
        if mod(n_var,2) ~= 0
            [pop_delta, pop_coeff] = decode_population(data,motion);
            pop = [pop_delta, pop_coeff];
        else
            [pop_omega, pop_delta, pop_coeff] = decode_population(data, motion);
            pop = [pop_omega, pop_delta, pop_coeff];
        end
    elseif strcmpi(motion,'MultiFourier')
        if mod(n_var,2) ~= 0
          [pop_delta, pop_coeff] = decode_population(data,motion);
            pop = [pop_delta, pop_coeff];
        else
           [omega, pop_delta, pop_coeff] = decode_population(data,motion);
            pop = [omega,pop_delta, pop_coeff];
        end
    else
        cd(return_path);
        error('Wrong motion pattern!')
    end
end

function [P_avg, v_avg] = get_objectives(objectives)
    P_avg = objectives(:,1);
    v_avg = -objectives(:,2);
end


function plot_phi(pop, t_start, t_step, t_end, fig_delay, x_bound, y_bound, motion)
    
    global n_var n

%     hFig  = figure(55);
%     set(hFig, 'Position', [300 200 600 500])
    grid on
    time = t_start:t_step:t_end;
    
    for k = 1:size(pop,1)
        figure(4);
        hold off
        figure(5);
        hold off
        figure(6);
        hold off
        for i = 1:9
            phi  = [];
            for t = time
                if strcmpi(motion,'Lateral') || strcmpi(motion,'Eel')
                    phi(end+1) = gait_function(pop, k, t, i);
%                 elseif strcmpi(motion,'Modified')
%                     phi(end+1) = modified_function(pop, k , t, i);
                elseif strcmpi(motion,'Modified')
                    phi(end+1) = modified1_function(pop, k , t, i);
                elseif strcmpi(motion,'FourierSeries_OLD')
                    phi(end+1) = fourier_function_no_omega(pop, k, t, i);
                elseif strcmpi(motion,'FourierSeries')
                    phi(end+1) = fourier_function(pop, k, t, i);
                elseif strcmpi(motion,'MultiFourier')
                    phi(end+1) = multifourier_function(pop, k, t, i);
                end          
            end
            if strcmpi(motion,'Modified') || strcmpi(motion,'MultiFourier')
                if(i == 1 || i == 2 || i == 8 || i == 9)

                    hFig = figure(4);
%                     set(hFig, 'Position', [300 200 600 500])
                    plot(time, phi*180/pi)
                    hold on;
                    xlabel('$t$','FontSize',14,'interpreter','latex');
                    ylabel('$\phi_i^*$','FontSize',14,'interpreter','latex');
                    le = legend({'$i=1$','$i=2$','$i=8$','$i=9$'},'Location', 'ne');
                    set(le,'FontSize',12,'interpreter','latex');
                elseif (i == 4 || i == 5 || i == 6)
                    hFig = figure(5);
%                     set(hFig, 'Position', [300 200 600 500])

                    plot(time, phi*180/pi)
                    hold on;
                    xlabel('$t$','FontSize',14,'interpreter','latex');
                    ylabel('$\phi_i^*$','FontSize',14,'interpreter','latex');
                    le = legend({'$i=4$','$i=5$','$i=6$'},'Location', 'ne');
                    set(le,'FontSize',12,'interpreter','latex');
                elseif (i == 3 || i == 7)               
                    hFig = figure(6);
%                     set(hFig, 'Position', [300 200 600 500])

                    plot(time, phi*180/pi);
                    hold on;
                    xlabel('$t$','FontSize',14,'interpreter','latex');
                    ylabel('$\phi_i^*$','FontSize',14,'interpreter','latex');
                    le = legend({'$i=3$','$i=7$'},'Location', 'ne');
                    set(le,'FontSize',12,'interpreter','latex');
                end
            else
                hFig = figure(7);
                set(hFig, 'Position', [300 200 600 500])
                hold on
                plot(time, phi*180/pi);
                xlabel('$t$','FontSize',14,'interpreter','latex');
                ylabel('$\phi_i^*$','FontSize',14,'interpreter','latex');
%                 le = legend({'$g_3$','$g_7$'},'Location', 'ne');
%                 set(le,'FontSize',12,'interpreter','latex')
            end
            xlim(x_bound);
            ylim(y_bound);
            grid on
%             pause(0.2)
        end
        pause(fig_delay);
    end
end

% function phi = new_modified_function(pop, k , t, i)
%     
%     alpha   = pop(k,1);
%     omega   = pop(k,2);
%     delta   = pop(k,3);
%     g_scale = pop(k,3+i);
%     phi     = g_scale*alpha*sin(omega*t + (i-1)*delta);
%     
% end

function phi = gait_function(pop, k , t, i)
    
    global omega_max n_var
        
    if mod(n_var,2) ~= 0
        alpha   = pop(k,1)*pi/180;
        omega   = pop(k,2)*pi/180;
        delta   = pop(k,3)*pi/180;
        phi     = alpha*sin(omega*t + (i-1)*delta);
    else 
        alpha   = pop(k,1)*pi/180;
        omega   = omega_max *pi/180;
        delta   = pop(k,2)*pi/180;
        phi     = alpha*sin(omega*t + (i-1)*delta);
    end
end

function phi = modified_function(pop, k , t, i)
    
    alpha   = pop(k,1)*pi/180;
    omega   = pop(k,2)*pi/180;
    delta   = pop(k,3)*pi/180;
    g_scale = pop(k,3+i);
    phi     = g_scale*alpha*sin(omega*t + (i-1)*delta);
    
end

function phi = modified1_function(pop, k , t, i)
    
    global omega_max n_var n

    if mod(n_var-(n-1),2) ~= 0
        omega   = omega_max * pi/180;
        delta   = pop(k,1)*pi/180;
        g_scale = pop(k,1+i)*pi/180;
        phi     = g_scale*sin(omega*t + (i-1)*delta);
    else 
        omega   = pop(k,1)*pi/180;
        delta   = pop(k,2)*pi/180;
        g_scale = pop(k,2+i)*pi/180;
        phi     = g_scale*sin(omega*t + (i-1)*delta);
    end
end

function phi = fourier_function_no_omega(pop, k , t, i)

    global omega_max
    
    omega = omega_max * pi/180;
    delta = pop(k,1)*pi/180;
    four_coeff = pop(k,2:end)*pi/180;
    
    a_coeff = four_coeff(1:ceil(end/2));
    b_coeff = four_coeff(ceil(end/2)+1:end);
    n_order = 1:(numel(four_coeff))/2;
    
    L       = numel(n_order)/omega;
    
    f1      = @(n,t,offset)(cos((n/L)*t + offset));
    f2      = @(n,t,offset)(sin((n/L)*t + offset));
    phi     = sum(a_coeff.*f1(n_order,t,(i-1)*delta) + b_coeff.*f2(n_order,t,(i-1)*delta)); 
    
end

function phi = fourier_function(pop, k, t, i)
    
    global omega_max n_var
    
    if mod(n_var,2) ~= 0
        omega = omega_max * pi/180;
        delta = pop(k,1)*pi/180;
        four_coeff = pop(k,2:end)*pi/180;
    else
        omega = pop(k,1)*pi/180;
        delta = pop(k,2)*pi/180;
        four_coeff = pop(k,3:end)*pi/180;
    end
 
    a_coeff = four_coeff(1:ceil(end/2));
    b_coeff = four_coeff(ceil(end/2)+1:end);
    n_order = 1:(numel(four_coeff))/2;

    L       = numel(n_order)/omega;
%     L       = 1;
    f1      = @(n,t,offset)(cos((n/L)*t + offset));
    f2      = @(n,t,offset)(sin((n/L)*t + offset));
    phi     = sum(a_coeff.*f1(n_order,t,(i-1)*delta) + b_coeff.*f2(n_order,t,(i-1)*delta)); 
end

function phi = multifourier_function(pop, k , t, i)

    global omega_max n_var n
    
    if mod(n_var,2) ~= 0
        omega = omega_max * pi/180;   
        delta = pop(k,1)*pi/180;
        four_coeff = pop(k,2:end)*pi/180;
    else
        omega = pop(k,1)*pi/180;   
        delta = pop(k,2)*pi/180;
        four_coeff = pop(k,3:end)*pi/180;
    end

    a = four_coeff(1:ceil(end/2));
    b = four_coeff(ceil(end/2)+1:end);
    
    n_coeff = numel(four_coeff)/((n-1)*2);
    n_order = 1:(n_coeff);

%     L       = 1;
    L       = numel(n_order)/omega;
    f1      = @(n,t,offset)(cos((n/L)*t + offset));
    f2      = @(n,t,offset)(sin((n/L)*t + offset)); 
    
    phi = sum(a(1+(i-1)*n_coeff:i*n_coeff).*f1(n_order,t,(i-1)*delta) + b(1+(i-1)*n_coeff:i*n_coeff).*f2(n_order,t,(i-1)*delta));
end
