function [regress, obj] = regress_model(motion)
    
    return_path = pwd;
    
    nr_of_poly = 16;

    obj = get_pareto_front(motion);
    v_func = compute_poly_func(obj, nr_of_poly);
    [beta_val, v_bound] = get_pls_beta(motion);
    
   v_func(obj(:,1))
   
    regress = @(P_avg)(compute_regress_model(P_avg, v_func, beta_val, v_bound));
    warning('off','last')
    cd(return_path);

    function [objectives] = get_pareto_front(motion)
        cd('../pareto_fronts');
        objectives = [];

        load_file = ['pareto','_',motion];
        pareto_front = load(load_file);
        objectives = pareto_front.obj;
    end

    function [v_func] = compute_poly_func(obj, n_p)
        p = polyfit(obj(:,1),obj(:,2),n_p);
        v_func = @(velocity)(polyval(p,velocity));
    end
    
    function [beta_val, v_bound] = get_pls_beta(motion)
        cd('../beta_coefficients');

        if strcmpi(motion,'lateral')
            pls1 = load('lateral_beta1');
            pls2 = load('lateral_beta2');
            beta1 = pls1.beta1;
            beta2 = pls2.beta2;
            beta_val = {beta1, beta2};
            v_bound = [2.0, 8.4];
        elseif strcmpi(motion,'Eel')
            pls1 = load('eel_beta1');
            pls2 = load('eel_beta2');
            beta1 = pls1.beta;
            beta2 = pls2.beta;
            beta_val = {beta1, beta2};
            v_bound = [2.0, 6.5];
        elseif strcmpi(motion,'Modified')
            pls = load('modified_beta');
            beta = pls.beta;
            beta_val = {beta};
            v_bound = 9.3;
        elseif strcmpi(motion,'FourierSeries1')
            pls1 = load('fourier1_beta1');
            pls2 = load('fourier1_beta2');
            beta1 = pls1.beta;
            beta2 = pls2.beta;
            beta_val = {beta1, beta2};
            v_bound = [2.25, 8.3];
        elseif strcmpi(motion,'FourierSeries3')
            pls1 = load('fourier3_beta1');
            pls2 = load('fourier3_beta2');
%             pls3 = load('fourier3_beta3');
%             pls4 = load('fourier3_beta4');
            beta1 = pls1.beta;
            beta2 = pls2.beta;
%             beta3 = pls3.beta;
%             beta4 = pls4.beta;
%             beta_val = {beta1, beta2, beta3, beta4};
            beta_val = {beta1, beta2};
            v_bound = [6.45, 7.8];
        elseif strcmpi(motion,'MultiFourier')
             pls1 = load('multifourier_beta1');
             beta1 = pls1.beta;
             beta_val = {beta1};
             v_bound = [8.8];
        end
    end  

    function [decision_var] = compute_regress_model(P_avg, v_func, beta_val, v_bound)
        
        
        v_avg = v_func(P_avg);
        
        decision_var = [];
        for i = 1:numel(v_avg)
            response = [];
            if v_avg(i) < 0 && v_avg(i) > max(v_bound)
                error('\n%s\n','Invalid value!!!')
            end

            for k = 1:numel(v_bound)
               if v_avg(i) <= v_bound(k)
                  response = [1,P_avg(i),v_avg(i)]*beta_val{k};
                  break;
               end
            end

            if isempty(response)
               [a,k] = min(abs(v_bound-v_avg(i)));
               response = [1,P_avg(i),v_avg(i)]*beta_val{k}; 
            end
            if response(1) >=0 && response(1) <= 90
                decision_var = [decision_var;response];
            end
        end
    end
end