import numpy as np

############################################## Gait limits ########################################################
# alpha_min   = 6   *(np.pi/180)
# alpha_max   = 32  *(np.pi/180)
# omega_min   = 10  *(np.pi/180)
# omega_max   = 120 *(np.pi/180)
# delta_min   = 35  *(np.pi/180)
# delta_max   = 50  *(np.pi/180)

alpha_min   = 0     *(np.pi/180)
alpha_max   = 90    *(np.pi/180)
omega_min   = 0     *(np.pi/180)
omega_max   = 210   *(np.pi/180)
delta_min   = 0     *(np.pi/180)
delta_max   = 90    *(np.pi/180)

# Fourier series
coeff_max   = 90    *(np.pi/180)
a_null_max  = 90    *(np.pi/180)

####################################### Number of objective functions #############################################
n_objectives = np.int32(2)

########################################### Length of each genes ##################################################
# Gene lengths
alpha_gen   = 7
omega_gen   = 7
delta_gen   = 7

# Modified motion
g_scale     = 7

# Fourier series
n_coefficients      = 2*1
# Gene length for each coefficients
fourier_gen_length  = 7
a_null              = 7

######################################### Chromosme representation ################################################
def chromosome_repr(eng, motion, with_omega):
    if motion == "Modified":
        nr_of_links = eng.getNumberOfLinks(nargout=1)
        # chrom = [alpha_gen, omega_gen, delta_gen] + [g_scale]*np.int16(nr_of_links-1)
        if with_omega:
            chrom = [omega_gen, delta_gen] + [g_scale]*np.int16(nr_of_links-1)
        else:
            chrom = [delta_gen] + [g_scale]*np.int16(nr_of_links-1)
        return np.array(chrom)
    if motion == "FourierSeries":
        # chrom = [omega_gen, delta_gen, a_null] + [fourier_gen_length]*n_coefficients
        if with_omega:
            chrom = [omega_gen, delta_gen] + [fourier_gen_length]*n_coefficients
        else:
            chrom = [delta_gen] + [fourier_gen_length]*n_coefficients
        return np.array(chrom)
    if motion == "MultiFourier":
        nr_of_links = eng.getNumberOfLinks(nargout=1)
        if with_omega:
            chrom = [omega_gen, delta_gen] + [fourier_gen_length]*np.int16(n_coefficients*(nr_of_links-1))
        else:
            chrom = [delta_gen] + [fourier_gen_length]*np.int16(n_coefficients*(nr_of_links-1))
        return np.array(chrom)
    else:
        if with_omega:
            return np.array([alpha_gen, omega_gen, delta_gen])
        else:
            return np.array([alpha_gen, delta_gen])

######################################### Upper and Lower bounds ##################################################
def L_BOUND(chromosome):
    return [0]*len(chromosome)

def U_BOUND(chromosome):
    return (np.power(2,chromosome)-1).tolist()

################################# Population size and number of generations #######################################
pop_size        = 500   #300
n_generations   = 300    #500

############################################# Genetic rates #######################################################
crossover_rate = 0.70
mutation_rate = 0.150
# crossover_rate = [0.8]
# crossover_rate = [0.6,0.7,0.8]          # Check params
# mutation_rate = [0.05,0.10,0.15]     # Check params

################################ Reference set used in optimization with HypE #####################################
reference = np.array([[100, -2],[0,0.05]]).T.tolist()

################################################ MOEA/D ###########################################################
nr_of_neighborhood_weight = 10
ref_point = [0, -1]                    # Reference point
