# MOEA
from Algorithms.NSGA2 import nsga2
from Algorithms.HypE import hype
from Algorithms.Utility.LoggerObject import LoggerObject
from Algorithms.main_interface import main_interface

from Algorithms.Utility.print_elapsed_time import time_to_text

# DEAP Library
from deap import algorithms
from deap import base
from deap import creator
from deap import tools

# Configuration File
import config as cg

import matlab.engine
import random
import array
import os

def random_int(low_b, upp_b):
    return [random.randint(x, y) for x,y in zip(low_b,upp_b)]

def select_moea(moea_name):
    for i in range(len(moea_name)):
        print("##\t",i+1,") ", moea_name[i])
    print("##")
    return int(input("== Select moea: "))

def select_motion_pattern(motion):
    print("\n## Snake robot motion patterns,")
    for i in range(len(motion)):
        print("##\t",i+1,") ", motion[i])
    print("##")
    return int(input("== Select motion pattern: "))

def optimize_with_omega():
    print("\n## Run optimization with omega?")
    print("##")
    repsonse =  input("== [Y/N]: ")
    while not (repsonse == 'Y' or repsonse == 'N'):
        repsonse = input("== [Y/N]: ")
    if repsonse == 'Y':
        return True
    else:
        return False

def print_moea_header(moea_name, motion):
    os.system('cls')
    print("###########################################" + \
            "###########################################")
    print("                Starting optimization with " + \
                        moea_name + " and " + motion)
    print("###########################################" + \
            "###########################################")

def check_GA_params(moea, moea_name, motion, eng, toolbox, chrom_repr, Low, Upp, result_dir, n, cx, mx, with_omega):
    elapsed_time = 0;
    for k in range(n):
        for c in cx:
            for m in mx:
                loggerObj = LoggerObject(moea_name, motion,
                                    "ga_result3.json", "ga_info3.txt", "tempData1.json", "ga_result3.zip", c, m)
                elapsed_time += main_interface(moea, moea_name, motion, eng, toolbox, chrom_repr, Low, Upp,
                                                                        loggerObj, result_dir,with_omega, False)
    print("\n############## GA RUN DONE!  ##############")
    time_text = time_to_text(elapsed_time)
    print("ELAPSED RUNTIME: " + time_text)
    print("###########################################\n")

def main(simulation_dir, result_dir):

    print("## Connecting to MATLAB engine...")
    os.chdir(simulation_dir)
    eng = matlab.engine.start_matlab()
    os.system("cls")

    creator.create("Fitness", base.Fitness, weights=(-1.0,-1.0))
    creator.create("Individual", array.array, typecode='I', fitness=creator.Fitness)

    moea          = [nsga2, hype]
    moea_name     = ["nsga-2", "HypE"]
    motion        = ["Lateral", "Eel", "Modified", "FourierSeries", "MultiFourier"]

    print("====================== Initialization ======================")
    print("## Optimization methods,")
    x             = select_moea(moea_name)
    y             = select_motion_pattern(motion)
    with_omega    = optimize_with_omega()

    if y > 1  or y <= len(motion):
        if x > 1 or x <= len(moea_name):

            print_moea_header(moea_name[x-1], motion[y-1])

            chrom_repr = cg.chromosome_repr(eng, motion[y-1], with_omega)
            Low = cg.L_BOUND(chrom_repr)
            Upp = cg.U_BOUND(chrom_repr)

            toolbox = base.Toolbox()
            toolbox.register("attr_uint", random_int, Low, Upp)
            toolbox.register("individual", tools.initIterate, creator.Individual, toolbox.attr_uint)
            toolbox.register("population", tools.initRepeat, list, toolbox.individual)

            if type(cg.mutation_rate) == type(1.0):
                redundancy_check = True;
                loggerObj = LoggerObject(moea_name[x-1], motion[y-1],
                                            "result1.json", "result_info1.txt", "tempData1.json", "result1.zip",
                                                                            cg.crossover_rate, cg.mutation_rate)
                results = main_interface(moea[x-1], moea_name[x-1], motion[y-1], eng, toolbox,
                                    chrom_repr, Low, Upp, loggerObj, result_dir, with_omega, redundancy_check)
            else:
                n_samples = 2
                check_GA_params(moea[x-1], moea_name[x-1], motion[y-1], eng, toolbox, chrom_repr, Low, Upp,
                                             result_dir, n_samples, cg.crossover_rate, cg.mutation_rate, with_omega)

        else:
            print("Wrong input in selecting an optimization moea")
    else:
        print("Wrong input in selecting a motion pattern")

if __name__ == "__main__":
    main("./Simulator/USR_extended_model", "../results/extended_model")
